@@ -4,17 +4,28 @@ pub mod consts {
 
 use std::rc::Rc;
 
+use crate::context::{DataType, ShaderType};
 use consts::Gl as InnerGl;
 
-pub type AttributeLocation = u32;
-pub type UniformLocation = u32;
-pub type Shader = u32;
-pub type Program = u32;
-pub type Buffer = u32;
-pub type Framebuffer = u32;
-pub type Texture = u32;
-pub type VertexArrayObject = u32;
+#[derive(Copy, Clone, Debug)]
+pub struct AttributeLocation(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct UniformLocation(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct Shader(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct Program(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct Buffer(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct Framebuffer(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct Texture(u32);
+#[derive(Copy, Clone, Debug)]
+pub struct VertexArrayObject(u32);
+
 pub type Sync = consts::types::GLsync;
+
 pub struct ActiveInfo {
     size: u32,
     type_: u32,
@@ -64,9 +75,9 @@ impl Context {
         }
     }
 
-    pub fn create_shader(&self, type_: u32) -> Option<Shader> {
-        let id = unsafe { self.inner.CreateShader(type_) };
-        Some(id)
+    pub fn create_shader(&self, type_: ShaderType) -> Option<Shader> {
+        let id = unsafe { self.inner.CreateShader(type_.to_const()) };
+        Some(Shader(id))
     }
 
     pub fn compile_shader(&self, source: &str, shader: &Shader) {
@@ -78,8 +89,8 @@ impl Context {
 
         unsafe {
             self.inner
-                .ShaderSource(*shader, 1, &c_str.as_ptr(), std::ptr::null());
-            self.inner.CompileShader(*shader);
+                .ShaderSource(shader.0, 1, &c_str.as_ptr(), std::ptr::null());
+            self.inner.CompileShader(shader.0);
         }
     }
 
@@ -87,7 +98,7 @@ impl Context {
         let mut len: consts::types::GLint = 0;
         unsafe {
             self.inner
-                .GetShaderiv(*shader, consts::INFO_LOG_LENGTH, &mut len);
+                .GetShaderiv(shader.0, consts::INFO_LOG_LENGTH, &mut len);
         }
 
         if len == 0 {
@@ -96,7 +107,7 @@ impl Context {
             let error = create_whitespace_cstring_with_len(len as usize);
             unsafe {
                 self.inner.GetShaderInfoLog(
-                    *shader,
+                    shader.0,
                     len,
                     std::ptr::null_mut(),
                     error.as_ptr() as *mut consts::types::GLchar,
@@ -108,26 +119,26 @@ impl Context {
 
     pub fn delete_shader(&self, shader: Option<&Shader>) {
         unsafe {
-            self.inner.DeleteShader(*shader.unwrap());
+            self.inner.DeleteShader(shader.unwrap().0);
         }
     }
 
     pub fn attach_shader(&self, program: &Program, shader: &Shader) {
         unsafe {
-            self.inner.AttachShader(*program, *shader);
+            self.inner.AttachShader(program.0, shader.0);
         }
     }
 
     pub fn detach_shader(&self, program: &Program, shader: &Shader) {
         unsafe {
-            self.inner.DetachShader(*program, *shader);
+            self.inner.DetachShader(program.0, shader.0);
         }
     }
 
     pub fn get_program_parameter(&self, program: &Program, pname: u32) -> u32 {
         let mut out = 0;
         unsafe {
-            self.inner.GetProgramiv(*program, pname, &mut out);
+            self.inner.GetProgramiv(program.0, pname, &mut out);
         }
         out as u32
     }
@@ -139,7 +150,7 @@ impl Context {
         let name = create_whitespace_cstring_with_len(length as usize);
         unsafe {
             self.inner.GetActiveAttrib(
-                *program,
+                program.0,
                 index,
                 length,
                 &mut length,
@@ -161,7 +172,7 @@ impl Context {
         let name = create_whitespace_cstring_with_len(length as usize);
         unsafe {
             self.inner.GetActiveUniform(
-                *program,
+                program.0,
                 index,
                 length,
                 &mut length,
@@ -181,12 +192,12 @@ impl Context {
         unsafe {
             self.inner.GenBuffers(1, &mut id);
         }
-        Some(id)
+        Some(Buffer(id))
     }
 
     pub fn delete_buffer(&self, buffer: &Buffer) {
         unsafe {
-            self.inner.DeleteBuffers(1, [*buffer].as_ptr());
+            self.inner.DeleteBuffers(1, [buffer.0].as_ptr());
         }
     }
 
@@ -205,13 +216,13 @@ impl Context {
                 println!("{}", current);
                 panic!();
             }
-            self.inner.BindBufferBase(target, index, *buffer);
+            self.inner.BindBufferBase(target, index, buffer.0);
         }
     }
 
     pub fn bind_buffer(&self, target: u32, buffer: &Buffer) {
         unsafe {
-            self.inner.BindBuffer(target, *buffer);
+            self.inner.BindBuffer(target, buffer.0);
         }
     }
 
@@ -223,12 +234,12 @@ impl Context {
 
     pub fn get_uniform_block_index(&self, program: &Program, name: &str) -> u32 {
         let c_str = std::ffi::CString::new(name).unwrap();
-        unsafe { self.inner.GetUniformBlockIndex(*program, c_str.as_ptr()) }
+        unsafe { self.inner.GetUniformBlockIndex(program.0, c_str.as_ptr()) }
     }
 
     pub fn uniform_block_binding(&self, program: &Program, location: u32, index: u32) {
         unsafe {
-            self.inner.UniformBlockBinding(*program, location, index);
+            self.inner.UniformBlockBinding(program.0, location, index);
         }
     }
 
@@ -254,6 +265,17 @@ impl Context {
         }
     }
 
+    pub fn buffer_data_u16(&self, target: u32, data: &[u16], usage: u32) {
+        unsafe {
+            self.inner.BufferData(
+                target,
+                (data.len() * std::mem::size_of::<u16>()) as consts::types::GLsizeiptr, // size of data in bytes
+                data.as_ptr() as *const consts::types::GLvoid, // pointer to data
+                usage,
+            );
+        }
+    }
+
     pub fn buffer_data_u32(&self, target: u32, data: &[u32], usage: u32) {
         unsafe {
             self.inner.BufferData(
@@ -281,28 +303,28 @@ impl Context {
         unsafe {
             self.inner.GenVertexArrays(1, &mut id);
         }
-        Some(id)
+        Some(VertexArrayObject(id))
     }
 
     pub fn bind_vertex_array(&self, array: &VertexArrayObject) {
         unsafe {
-            self.inner.BindVertexArray(*array);
+            self.inner.BindVertexArray(array.0);
         }
     }
 
     pub fn create_program(&self) -> Program {
-        unsafe { self.inner.CreateProgram() }
+        unsafe { Program(self.inner.CreateProgram()) }
     }
 
     pub fn link_program(&self, program: &Program) -> bool {
         unsafe {
-            self.inner.LinkProgram(*program);
+            self.inner.LinkProgram(program.0);
         }
 
         let mut success: consts::types::GLint = 1;
         unsafe {
             self.inner
-                .GetProgramiv(*program, consts::LINK_STATUS, &mut success);
+                .GetProgramiv(program.0, consts::LINK_STATUS, &mut success);
         }
         success == 1
     }
@@ -311,7 +333,7 @@ impl Context {
         let mut len: consts::types::GLint = 0;
         unsafe {
             self.inner
-                .GetProgramiv(*program, consts::INFO_LOG_LENGTH, &mut len);
+                .GetProgramiv(program.0, consts::INFO_LOG_LENGTH, &mut len);
         }
 
         if len == 0 {
@@ -320,7 +342,7 @@ impl Context {
             let error = create_whitespace_cstring_with_len(len as usize);
             unsafe {
                 self.inner.GetProgramInfoLog(
-                    *program,
+                    program.0,
                     len,
                     std::ptr::null_mut(),
                     error.as_ptr() as *mut consts::types::GLchar,
@@ -332,7 +354,7 @@ impl Context {
 
     pub fn use_program(&self, program: &Program) {
         unsafe {
-            self.inner.UseProgram(*program);
+            self.inner.UseProgram(program.0);
         }
     }
 
@@ -344,29 +366,29 @@ impl Context {
 
     pub fn delete_program(&self, program: &Program) {
         unsafe {
-            self.inner.DeleteProgram(*program);
+            self.inner.DeleteProgram(program.0);
         }
     }
 
     pub fn get_attrib_location(&self, program: &Program, name: &str) -> Option<AttributeLocation> {
         let c_str = std::ffi::CString::new(name).unwrap();
-        let location = unsafe { self.inner.GetAttribLocation(*program, c_str.as_ptr()) };
+        let location = unsafe { self.inner.GetAttribLocation(program.0, c_str.as_ptr()) };
         if location == -1 {
             None
         } else {
-            Some(location as AttributeLocation)
+            Some(AttributeLocation(location as _))
         }
     }
 
     pub fn enable_vertex_attrib_array(&self, location: AttributeLocation) {
         unsafe {
-            self.inner.EnableVertexAttribArray(location);
+            self.inner.EnableVertexAttribArray(location.0);
         }
     }
 
     pub fn disable_vertex_attrib_array(&self, location: AttributeLocation) {
         unsafe {
-            self.inner.DisableVertexAttribArray(location);
+            self.inner.DisableVertexAttribArray(location.0);
         }
     }
 
@@ -374,19 +396,19 @@ impl Context {
         &self,
         location: AttributeLocation,
         size: u32,
-        data_type: u32,
+        data_type: DataType,
         normalized: bool,
         stride: u32,
         offset: u32,
     ) {
         unsafe {
             self.inner.VertexAttribPointer(
-                location as consts::types::GLuint, // index of the generic vertex attribute
+                location.0 as consts::types::GLuint, // index of the generic vertex attribute
                 size as consts::types::GLint, // the number of components per generic vertex attribute
-                data_type as consts::types::GLenum, // data type
+                data_type.to_const(),         // data type
                 normalized as consts::types::GLboolean, // normalized (int-to-float conversion)
-                byte_size_for_type(data_type, stride) as consts::types::GLint, // stride (byte offset between consecutive attributes)
-                byte_size_for_type(data_type, offset) as *const consts::types::GLvoid, // offset of the first component
+                (stride * data_type.byte_size()) as consts::types::GLint, // stride (byte offset between consecutive attributes)
+                (offset * data_type.byte_size()) as *const consts::types::GLvoid, // offset of the first component
             );
         }
     }
@@ -394,7 +416,7 @@ impl Context {
     pub fn vertex_attrib_divisor(&self, location: AttributeLocation, divisor: u32) {
         unsafe {
             self.inner.VertexAttribDivisor(
-                location as consts::types::GLuint,
+                location.0 as consts::types::GLuint,
                 divisor as consts::types::GLuint,
             );
         }
@@ -402,62 +424,62 @@ impl Context {
 
     pub fn get_uniform_location(&self, program: &Program, name: &str) -> Option<UniformLocation> {
         let c_str = std::ffi::CString::new(name).unwrap();
-        let location = unsafe { self.inner.GetUniformLocation(*program, c_str.as_ptr()) };
+        let location = unsafe { self.inner.GetUniformLocation(program.0, c_str.as_ptr()) };
         if location == -1 {
             None
         } else {
-            Some(location as UniformLocation)
+            Some(UniformLocation(location as _))
         }
     }
 
     pub fn uniform1i(&self, location: &UniformLocation, data: i32) {
         unsafe {
-            self.inner.Uniform1i(*location as i32, data);
+            self.inner.Uniform1i(location.0 as i32, data);
         }
     }
 
     pub fn uniform1f(&self, location: &UniformLocation, data: f32) {
         unsafe {
-            self.inner.Uniform1f(*location as i32, data);
+            self.inner.Uniform1f(location.0 as i32, data);
         }
     }
 
     pub fn uniform2fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
-            self.inner.Uniform2fv(*location as i32, 1, data.as_ptr());
+            self.inner.Uniform2fv(location.0 as i32, 1, data.as_ptr());
         }
     }
 
     pub fn uniform3fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
-            self.inner.Uniform3fv(*location as i32, 1, data.as_ptr());
+            self.inner.Uniform3fv(location.0 as i32, 1, data.as_ptr());
         }
     }
 
     pub fn uniform4fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
-            self.inner.Uniform4fv(*location as i32, 1, data.as_ptr());
+            self.inner.Uniform4fv(location.0 as i32, 1, data.as_ptr());
         }
     }
 
     pub fn uniform_matrix2fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
             self.inner
-                .UniformMatrix2fv(*location as i32, 1, consts::FALSE, data.as_ptr());
+                .UniformMatrix2fv(location.0 as i32, 1, consts::FALSE, data.as_ptr());
         }
     }
 
     pub fn uniform_matrix3fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
             self.inner
-                .UniformMatrix3fv(*location as i32, 1, consts::FALSE, data.as_ptr());
+                .UniformMatrix3fv(location.0 as i32, 1, consts::FALSE, data.as_ptr());
         }
     }
 
     pub fn uniform_matrix4fv(&self, location: &UniformLocation, data: &[f32]) {
         unsafe {
             self.inner
-                .UniformMatrix4fv(*location as i32, 1, consts::FALSE, data.as_ptr());
+                .UniformMatrix4fv(location.0 as i32, 1, consts::FALSE, data.as_ptr());
         }
     }
 
@@ -473,12 +495,12 @@ impl Context {
         unsafe {
             self.inner.GenFramebuffers(1, &mut id);
         }
-        Some(id)
+        Some(Framebuffer(id))
     }
 
     pub fn bind_framebuffer(&self, target: u32, framebuffer: Option<&Framebuffer>) {
         let id = match framebuffer {
-            Some(fb) => *fb,
+            Some(fb) => fb.0,
             None => 0,
         };
         unsafe {
@@ -488,7 +510,7 @@ impl Context {
 
     pub fn delete_framebuffer(&self, framebuffer: Option<&Framebuffer>) {
         let id = match framebuffer {
-            Some(fb) => fb,
+            Some(fb) => &fb.0,
             None => &0,
         };
         unsafe {
@@ -554,9 +576,9 @@ impl Context {
         }
     }
 
-    pub fn viewport(&self, x: i32, y: i32, width: usize, height: usize) {
+    pub fn viewport(&self, x: i32, y: i32, width: i32, height: i32) {
         unsafe {
-            self.inner.Viewport(x, y, width as i32, height as i32);
+            self.inner.Viewport(x, y, width, height);
         }
     }
 
@@ -621,6 +643,12 @@ impl Context {
         }
     }
 
+    pub fn scissor(&self, x: i32, y: i32, width: i32, height: i32) {
+        unsafe {
+            self.inner.Scissor(x, y, width, height);
+        }
+    }
+
     pub fn depth_func(&self, func: u32) {
         unsafe {
             self.inner.DepthFunc(func);
@@ -653,7 +681,7 @@ impl Context {
         unsafe {
             self.inner.GenTextures(1, &mut id);
         }
-        Some(id)
+        Some(Texture(id))
     }
 
     pub fn active_texture(&self, texture: u32) {
@@ -664,7 +692,7 @@ impl Context {
 
     pub fn bind_texture(&self, target: u32, texture: &Texture) {
         unsafe {
-            self.inner.BindTexture(target, *texture);
+            self.inner.BindTexture(target, texture.0);
         }
     }
 
@@ -723,7 +751,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
     ) {
         unsafe {
             self.inner.TexImage2D(
@@ -734,7 +762,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 std::ptr::null() as *const consts::types::GLvoid,
             );
         }
@@ -749,7 +777,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u8],
     ) {
         unsafe {
@@ -761,7 +789,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 pixels.as_ptr() as *const consts::types::GLvoid,
             );
         }
@@ -776,7 +804,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u8],
     ) {
         unsafe {
@@ -788,7 +816,7 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 pixels.as_ptr() as *const consts::types::GLvoid,
             );
         }
@@ -803,7 +831,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[f32],
     ) {
         unsafe {
@@ -815,7 +843,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 pixels.as_ptr() as *const consts::types::GLvoid,
             );
         }
@@ -830,7 +858,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[f32],
     ) {
         unsafe {
@@ -842,7 +870,34 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
+                pixels.as_ptr() as *const consts::types::GLvoid,
+            );
+        }
+    }
+
+    pub fn tex_sub_image_2d_with_u32_data(
+        &self,
+        target: u32,
+        level: u32,
+        x_offset: u32,
+        y_offset: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        pixels: &[u32],
+    ) {
+        unsafe {
+            self.inner.TexSubImage2D(
+                target,
+                level as i32,
+                x_offset as i32,
+                y_offset as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
                 pixels.as_ptr() as *const consts::types::GLvoid,
             );
         }
@@ -857,7 +912,7 @@ impl Context {
         height: u32,
         depth: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
     ) {
         unsafe {
             self.inner.TexImage3D(
@@ -869,7 +924,7 @@ impl Context {
                 depth as i32,
                 0,
                 format,
-                data_type,
+                data_type.to_const(),
                 std::ptr::null() as *const consts::types::GLvoid,
             );
         }
@@ -885,7 +940,7 @@ impl Context {
         depth: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u16],
     ) {
         unsafe {
@@ -898,7 +953,7 @@ impl Context {
                 depth as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 pixels.as_ptr() as *const consts::types::GLvoid,
             );
         }
@@ -912,7 +967,7 @@ impl Context {
 
     pub fn delete_texture(&self, texture: &Texture) {
         unsafe {
-            self.inner.DeleteTextures(1, texture);
+            self.inner.DeleteTextures(1, &texture.0);
         }
     }
 
@@ -926,7 +981,7 @@ impl Context {
     ) {
         unsafe {
             self.inner
-                .FramebufferTexture2D(target, attachment, textarget, *texture, level as i32);
+                .FramebufferTexture2D(target, attachment, textarget, texture.0, level as i32);
         }
     }
 
@@ -942,7 +997,7 @@ impl Context {
             self.inner.FramebufferTextureLayer(
                 target,
                 attachment,
-                *texture,
+                texture.0,
                 level as i32,
                 layer as i32,
             );
@@ -970,13 +1025,13 @@ impl Context {
         }
     }
 
-    pub fn draw_elements(&self, mode: u32, count: u32, data_type: u32, offset: u32) {
+    pub fn draw_elements(&self, mode: u32, count: u32, data_type: DataType, offset: u32) {
         unsafe {
             self.inner.DrawElements(
                 mode as consts::types::GLenum,
                 count as consts::types::GLint, // number of indices to be rendered
-                data_type as consts::types::GLenum,
-                byte_size_for_type(data_type, offset) as *const consts::types::GLvoid, // starting index in the enabled arrays
+                data_type.to_const(),
+                (offset * data_type.byte_size()) as *const consts::types::GLvoid, // starting index in the enabled arrays
             );
         }
     }
@@ -985,7 +1040,7 @@ impl Context {
         &self,
         mode: u32,
         count: u32,
-        data_type: u32,
+        data_type: DataType,
         offset: u32,
         instance_count: u32,
     ) {
@@ -993,8 +1048,8 @@ impl Context {
             self.inner.DrawElementsInstanced(
                 mode as consts::types::GLenum,
                 count as consts::types::GLint, // number of indices to be rendered
-                data_type as consts::types::GLenum,
-                byte_size_for_type(data_type, offset) as *const consts::types::GLvoid, // starting index in the enabled arrays
+                data_type.to_const(),
+                (offset * data_type.byte_size()) as *const consts::types::GLvoid, // starting index in the enabled arrays
                 instance_count as consts::types::GLint,
             );
         }
@@ -1007,7 +1062,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
     ) {
         unsafe {
             self.inner.ReadPixels(
@@ -1016,7 +1071,7 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 0 as *mut consts::types::GLvoid,
             );
         }
@@ -1029,7 +1084,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         dst_data: &mut [u8],
     ) {
         unsafe {
@@ -1039,7 +1094,7 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 dst_data.as_ptr() as *mut consts::types::GLvoid,
             )
         }
@@ -1052,7 +1107,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         dst_data: &mut [f32],
     ) {
         unsafe {
@@ -1062,7 +1117,30 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
+                dst_data.as_ptr() as *mut consts::types::GLvoid,
+            )
+        }
+    }
+
+    pub fn read_pixels_with_u32_data(
+        &self,
+        x: u32,
+        y: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        dst_data: &mut [u32],
+    ) {
+        unsafe {
+            self.inner.ReadPixels(
+                x as i32,
+                y as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
                 dst_data.as_ptr() as *mut consts::types::GLvoid,
             )
         }
@@ -1098,10 +1176,25 @@ fn create_whitespace_cstring_with_len(len: usize) -> std::ffi::CString {
     unsafe { std::ffi::CString::from_vec_unchecked(buffer) }
 }
 
-fn byte_size_for_type(data_type: u32, count: u32) -> u32 {
-    match data_type {
-        consts::FLOAT => count * std::mem::size_of::<f32>() as u32,
-        consts::UNSIGNED_INT => count * std::mem::size_of::<u32>() as u32,
-        _ => 0,
+impl ShaderType {
+    fn to_const(&self) -> u32 {
+        match self {
+            ShaderType::Vertex => consts::VERTEX_SHADER,
+            ShaderType::Fragment => consts::FRAGMENT_SHADER,
+        }
+    }
+}
+
+impl DataType {
+    fn to_const(&self) -> u32 {
+        match self {
+            DataType::Float => consts::FLOAT,
+            DataType::Byte => consts::BYTE,
+            DataType::UnsignedByte => consts::UNSIGNED_BYTE,
+            DataType::Short => consts::SHORT,
+            DataType::UnsignedShort => consts::UNSIGNED_SHORT,
+            DataType::Int => consts::INT,
+            DataType::UnsignedInt => consts::UNSIGNED_INT,
+        }
     }
 }
