@@ -4,6 +4,7 @@ use web_sys::WebGl2RenderingContext as InnerGl;
 pub type consts = InnerGl;
 
 pub type AttributeLocation = u32;
+use crate::context::{DataType, ShaderType};
 pub use web_sys::WebGlActiveInfo as ActiveInfo;
 pub use web_sys::WebGlBuffer as Buffer;
 pub use web_sys::WebGlFramebuffer as Framebuffer;
@@ -55,6 +56,20 @@ impl Context {
         self.inner.buffer_data_with_u8_array(target, data, usage)
     }
 
+    pub fn buffer_data_u16(&self, target: u32, data: &[u16], usage: u32) {
+        use wasm_bindgen::JsCast;
+        let memory_buffer = wasm_bindgen::memory()
+            .dyn_into::<js_sys::WebAssembly::Memory>()
+            .unwrap()
+            .buffer();
+        let data_location = data.as_ptr() as u32 / 2;
+        let array = js_sys::Uint16Array::new(&memory_buffer)
+            .subarray(data_location, data_location + data.len() as u32);
+
+        self.inner
+            .buffer_data_with_array_buffer_view(target, &array, usage);
+    }
+
     pub fn buffer_data_u32(&self, target: u32, data: &[u32], usage: u32) {
         use wasm_bindgen::JsCast;
         let memory_buffer = wasm_bindgen::memory()
@@ -83,6 +98,10 @@ impl Context {
             .buffer_data_with_array_buffer_view(target, &array, usage);
     }
 
+    pub fn create_shader(&self, type_: ShaderType) -> Option<Shader> {
+        self.inner.create_shader(type_.to_const())
+    }
+
     pub fn compile_shader(&self, source: &str, shader: &Shader) {
         let header = "#version 300 es\nprecision highp float;\nprecision highp int;\nprecision highp sampler2DArray;\n";
         let s: &str = &[header, source].concat();
@@ -160,7 +179,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
     ) {
         self.inner
             .tex_image_2d_with_i32_and_i32_and_i32_and_format_and_type_and_opt_u8_array(
@@ -171,7 +190,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 None,
             )
             .unwrap();
@@ -186,7 +205,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u8],
     ) {
         self.inner
@@ -198,7 +217,7 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(pixels),
             )
             .unwrap();
@@ -213,7 +232,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u8],
     ) {
         self.inner
@@ -225,7 +244,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(pixels),
             )
             .unwrap();
@@ -240,7 +259,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[f32],
     ) {
         use wasm_bindgen::JsCast;
@@ -261,7 +280,43 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
+                Some(&array),
+            )
+            .unwrap();
+    }
+
+    pub fn tex_sub_image_2d_with_u32_data(
+        &self,
+        target: u32,
+        level: u32,
+        x_offset: u32,
+        y_offset: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        pixels: &[u32],
+    ) {
+        use wasm_bindgen::JsCast;
+        let memory_buffer = wasm_bindgen::memory()
+            .dyn_into::<js_sys::WebAssembly::Memory>()
+            .unwrap()
+            .buffer();
+        let data_location = pixels.as_ptr() as u32 / 4;
+        let array = js_sys::Uint32Array::new(&memory_buffer)
+            .subarray(data_location, data_location + pixels.len() as u32);
+
+        self.inner
+            .tex_sub_image_2d_with_i32_and_i32_and_u32_and_type_and_opt_array_buffer_view(
+                target,
+                level as i32,
+                x_offset as i32,
+                y_offset as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
                 Some(&array),
             )
             .unwrap();
@@ -276,7 +331,7 @@ impl Context {
         height: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[f32],
     ) {
         use wasm_bindgen::JsCast;
@@ -297,7 +352,7 @@ impl Context {
                 height as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(&array),
             )
             .unwrap();
@@ -313,7 +368,7 @@ impl Context {
         depth: u32,
         border: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         pixels: &[u16],
     ) {
         use wasm_bindgen::JsCast;
@@ -335,7 +390,7 @@ impl Context {
                 depth as i32,
                 border as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(&array),
             )
             .unwrap();
@@ -382,7 +437,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         dst_data: &mut [u8],
     ) {
         self.inner
@@ -392,7 +447,7 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(dst_data),
             )
             .unwrap()
@@ -405,7 +460,7 @@ impl Context {
         width: u32,
         height: u32,
         format: u32,
-        data_type: u32,
+        data_type: DataType,
         dst_data: &mut [f32],
     ) {
         use wasm_bindgen::JsCast;
@@ -423,14 +478,41 @@ impl Context {
                 width as i32,
                 height as i32,
                 format,
-                data_type,
+                data_type.to_const(),
                 Some(&array),
             )
             .unwrap();
     }
 
-    pub fn viewport(&self, x: i32, y: i32, width: usize, height: usize) {
-        self.inner.viewport(x, y, width as i32, height as i32);
+    pub fn read_pixels_with_u32_data(
+        &self,
+        x: u32,
+        y: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        dst_data: &mut [u32],
+    ) {
+        use wasm_bindgen::JsCast;
+        let memory_buffer = wasm_bindgen::memory()
+            .dyn_into::<js_sys::WebAssembly::Memory>()
+            .unwrap()
+            .buffer();
+        let data_location = dst_data.as_ptr() as u32 / 4;
+        let array = js_sys::Uint32Array::new(&memory_buffer)
+            .subarray(data_location, data_location + dst_data.len() as u32);
+        self.inner
+            .read_pixels_with_opt_array_buffer_view(
+                x as i32,
+                y as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
+                Some(&array),
+            )
+            .unwrap();
     }
 
     pub fn get_attrib_location(&self, program: &Program, name: &str) -> Option<AttributeLocation> {
@@ -466,23 +548,23 @@ impl Context {
         );
     }
 
-    pub fn draw_elements(&self, mode: u32, count: u32, data_type: u32, offset: u32) {
+    pub fn draw_elements(&self, mode: u32, count: u32, data_type: DataType, offset: u32) {
         self.inner
-            .draw_elements_with_i32(mode, count as i32, data_type, offset as i32);
+            .draw_elements_with_i32(mode, count as i32, data_type.to_const(), offset as i32);
     }
 
     pub fn draw_elements_instanced(
         &self,
         mode: u32,
         count: u32,
-        data_type: u32,
+        data_type: DataType,
         offset: u32,
         instance_count: u32,
     ) {
         self.inner.draw_elements_instanced_with_i32(
             mode,
             count as i32,
-            data_type,
+            data_type.to_const(),
             offset as i32,
             instance_count as i32,
         );
@@ -586,7 +668,7 @@ impl Context {
         &self,
         location: AttributeLocation,
         size: u32,
-        data_type: u32,
+        data_type: DataType,
         normalized: bool,
         stride: u32,
         offset: u32,
@@ -594,10 +676,10 @@ impl Context {
         self.inner.vertex_attrib_pointer_with_i32(
             location,
             size as i32,
-            data_type,
+            data_type.to_const(),
             normalized,
-            byte_size_for_type(data_type, stride) as i32,
-            byte_size_for_type(data_type, offset) as i32,
+            (stride * data_type.byte_size()) as i32,
+            (offset * data_type.byte_size()) as i32,
         );
     }
 
@@ -637,10 +719,25 @@ impl std::ops::Deref for Context {
     }
 }
 
-pub fn byte_size_for_type(data_type: u32, count: u32) -> u32 {
-    match data_type {
-        consts::FLOAT => count * std::mem::size_of::<f32>() as u32,
-        consts::UNSIGNED_INT => count * std::mem::size_of::<u32>() as u32,
-        _ => 0,
+impl ShaderType {
+    fn to_const(&self) -> u32 {
+        match self {
+            ShaderType::Vertex => consts::VERTEX_SHADER,
+            ShaderType::Fragment => consts::FRAGMENT_SHADER,
+        }
+    }
+}
+
+impl DataType {
+    fn to_const(&self) -> u32 {
+        match self {
+            DataType::Float => consts::FLOAT,
+            DataType::Byte => consts::BYTE,
+            DataType::UnsignedByte => consts::UNSIGNED_BYTE,
+            DataType::Short => consts::SHORT,
+            DataType::UnsignedShort => consts::UNSIGNED_SHORT,
+            DataType::Int => consts::INT,
+            DataType::UnsignedInt => consts::UNSIGNED_INT,
+        }
     }
 }
