@@ -1,6 +1,5 @@
-use crate::context::{consts, Context};
-use crate::core::{Error::ProgramError, *};
-use crate::math::*;
+use crate::context::{consts, AttributeLocation, Context, ShaderType};
+use crate::core::*;
 use std::cell::RefCell;
 use std::collections::HashMap;
 
@@ -13,7 +12,7 @@ use std::collections::HashMap;
 pub struct Program {
     context: Context,
     id: crate::context::Program,
-    vertex_attributes: HashMap<String, u32>,
+    vertex_attributes: HashMap<String, AttributeLocation>,
     textures: RefCell<HashMap<String, u32>>,
     uniforms: HashMap<String, crate::context::UniformLocation>,
     uniform_blocks: RefCell<HashMap<String, (u32, u32)>>,
@@ -27,17 +26,13 @@ impl Program {
         context: &Context,
         vertex_shader_source: &str,
         fragment_shader_source: &str,
-    ) -> Result<Program, Error> {
+    ) -> Result<Program> {
         let vert_shader = context
-            .create_shader(consts::VERTEX_SHADER)
-            .ok_or(ProgramError {
-                message: "Unable to create Vertex shader object".to_string(),
-            })?;
+            .create_shader(ShaderType::Vertex)
+            .ok_or(CoreError::ShaderCreation)?;
         let frag_shader = context
-            .create_shader(consts::FRAGMENT_SHADER)
-            .ok_or(ProgramError {
-                message: "Unable to create Fragment shader object".to_string(),
-            })?;
+            .create_shader(ShaderType::Fragment)
+            .ok_or(CoreError::ShaderCreation)?;
         context.compile_shader(vertex_shader_source, &vert_shader);
         context.compile_shader(fragment_shader_source, &frag_shader);
 
@@ -47,17 +42,16 @@ impl Program {
         let success = context.link_program(&id);
 
         if !success {
-            let mut message = "Failed to compile shader program:\n".to_string();
             if let Some(log) = context.get_program_info_log(&id) {
-                message = format!("{}\nLink error: {}", message, log);
+                Err(CoreError::ShaderLink(log))?;
             }
             if let Some(log) = context.get_shader_info_log(&vert_shader) {
-                message = format!("{}\nVertex shader error: {}", message, log);
+                Err(CoreError::ShaderCompilation("vertex".to_string(), log))?;
             }
             if let Some(log) = context.get_shader_info_log(&frag_shader) {
-                message = format!("{}\nFragment shader error: {}", message, log);
+                Err(CoreError::ShaderCompilation("fragment".to_string(), log))?;
             }
-            return Err(Error::ProgramError { message });
+            unreachable!();
         }
 
         context.detach_shader(&id, &vert_shader);
@@ -101,7 +95,7 @@ impl Program {
     /// Send the given integer value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform int`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_int(&self, name: &str, data: &i32) -> Result<(), Error> {
+    pub fn use_uniform_int(&self, name: &str, data: &i32) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context.uniform1i(location, *data);
         self.context.unuse_program();
@@ -112,7 +106,7 @@ impl Program {
     /// Send the given float value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform float`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_float(&self, name: &str, data: &f32) -> Result<(), Error> {
+    pub fn use_uniform_float(&self, name: &str, data: &f32) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context.uniform1f(location, *data);
         self.context.unuse_program();
@@ -123,7 +117,7 @@ impl Program {
     /// Send the given [Vec2](crate::Vec2) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec2`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec2(&self, name: &str, data: &Vec2) -> Result<(), Error> {
+    pub fn use_uniform_vec2(&self, name: &str, data: &Vec2) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context.uniform2fv(location, &mut [data.x, data.y]);
         self.context.unuse_program();
@@ -134,7 +128,7 @@ impl Program {
     /// Send the given [Vec3](crate::Vec3) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec3`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec3(&self, name: &str, data: &Vec3) -> Result<(), Error> {
+    pub fn use_uniform_vec3(&self, name: &str, data: &Vec3) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform3fv(location, &mut [data.x, data.y, data.z]);
@@ -146,7 +140,7 @@ impl Program {
     /// Send the given [Vec4](crate::Vec4) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform vec4`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_vec4(&self, name: &str, data: &Vec4) -> Result<(), Error> {
+    pub fn use_uniform_vec4(&self, name: &str, data: &Vec4) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform4fv(location, &mut [data.x, data.y, data.z, data.w]);
@@ -158,7 +152,7 @@ impl Program {
     /// Send the given [Mat2](crate::Mat2) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat2`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat2(&self, name: &str, data: &Mat2) -> Result<(), Error> {
+    pub fn use_uniform_mat2(&self, name: &str, data: &Mat2) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix2fv(location, &mut data.to_slice());
@@ -170,7 +164,7 @@ impl Program {
     /// Send the given [Mat3](crate::Mat3) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat3`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat3(&self, name: &str, data: &Mat3) -> Result<(), Error> {
+    pub fn use_uniform_mat3(&self, name: &str, data: &Mat3) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix3fv(location, &mut data.to_slice());
@@ -182,7 +176,7 @@ impl Program {
     /// Send the given [Mat4](crate::Mat4) value to this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform mat4`, meaning it is uniformly available across all processing of vertices and fragments.
     ///
-    pub fn use_uniform_mat4(&self, name: &str, data: &Mat4) -> Result<(), Error> {
+    pub fn use_uniform_mat4(&self, name: &str, data: &Mat4) -> Result<()> {
         let location = self.get_uniform_location(name)?;
         self.context
             .uniform_matrix4fv(location, &mut data.to_slice());
@@ -190,203 +184,234 @@ impl Program {
         Ok(())
     }
 
-    fn get_uniform_location(&self, name: &str) -> Result<&crate::context::UniformLocation, Error> {
+    fn get_uniform_location(&self, name: &str) -> Result<&crate::context::UniformLocation> {
         self.set_used();
-        let loc = self.uniforms.get(name).ok_or_else(|| ProgramError {
-            message: format!(
-                "The uniform {} is sent to the shader but it is never used.",
-                name
-            ),
-        })?;
+        let loc = self
+            .uniforms
+            .get(name)
+            .ok_or_else(|| CoreError::UnusedUniform(name.to_string()))?;
         Ok(loc)
     }
 
     ///
-    /// Use the given [Texture2D](crate::Texture2D) in this shader program and associate it with the given named variable.
+    /// Use the given [Texture2D] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform sampler2D` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture(&self, texture: &impl Texture, texture_name: &str) -> Result<(), Error> {
-        let index = self.get_texture_index(texture_name);
+    pub fn use_texture(&self, name: &str, texture: &impl Texture) -> Result<()> {
+        let index = self.get_texture_index(name);
         texture.bind(index);
-        self.use_uniform_int(texture_name, &(index as i32))?;
+        self.use_uniform_int(name, &(index as i32))?;
         Ok(())
     }
 
     ///
-    /// Use the given [TextureArray](crate::TextureArray) in this shader program and associate it with the given named variable.
+    /// Use the given [TextureArray] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform sampler2DArray` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture_array(
-        &self,
-        texture: &impl TextureArray,
-        texture_name: &str,
-    ) -> Result<(), Error> {
-        let index = self.get_texture_index(texture_name);
+    pub fn use_texture_array(&self, name: &str, texture: &impl TextureArray) -> Result<()> {
+        let index = self.get_texture_index(name);
         texture.bind(index);
-        self.use_uniform_int(texture_name, &(index as i32))?;
+        self.use_uniform_int(name, &(index as i32))?;
         Ok(())
     }
 
     ///
-    /// Use the given [TextureCube](crate::TextureCube) in this shader program and associate it with the given named variable.
+    /// Use the given [TextureCube] in this shader program and associate it with the given named variable.
     /// The glsl shader variable must be of type `uniform samplerCube` and can only be accessed in the fragment shader.
     ///
-    pub fn use_texture_cube(
-        &self,
-        texture: &impl TextureCube,
-        texture_name: &str,
-    ) -> Result<(), Error> {
-        let index = self.get_texture_index(texture_name);
+    pub fn use_texture_cube(&self, name: &str, texture: &impl TextureCube) -> Result<()> {
+        let index = self.get_texture_index(name);
         texture.bind(index);
-        self.use_uniform_int(texture_name, &(index as i32))?;
+        self.use_uniform_int(name, &(index as i32))?;
         Ok(())
     }
 
-    fn get_texture_index(&self, texture_name: &str) -> u32 {
-        if !self.textures.borrow().contains_key(texture_name) {
+    fn get_texture_index(&self, name: &str) -> u32 {
+        if !self.textures.borrow().contains_key(name) {
             let mut map = self.textures.borrow_mut();
             let index = map.len() as u32;
-            map.insert(texture_name.to_owned(), index);
+            map.insert(name.to_owned(), index);
         };
-        self.textures.borrow().get(texture_name).unwrap().clone()
+        self.textures.borrow().get(name).unwrap().clone()
     }
 
-    pub fn use_uniform_block(&self, buffer: &UniformBuffer, block_name: &str) {
-        if !self.uniform_blocks.borrow().contains_key(block_name) {
+    ///
+    /// Use the given [UniformBuffer] in this shader program and associate it with the given named variable.
+    ///
+    pub fn use_uniform_block(&self, name: &str, buffer: &UniformBuffer) {
+        if !self.uniform_blocks.borrow().contains_key(name) {
             let mut map = self.uniform_blocks.borrow_mut();
-            let location = self.context.get_uniform_block_index(&self.id, block_name);
+            let location = self.context.get_uniform_block_index(&self.id, name);
             let index = map.len() as u32;
-            map.insert(block_name.to_owned(), (location, index));
+            map.insert(name.to_owned(), (location, index));
         };
-        let (location, index) = self
-            .uniform_blocks
-            .borrow()
-            .get(block_name)
-            .unwrap()
-            .clone();
+        let (location, index) = self.uniform_blocks.borrow().get(name).unwrap().clone();
         self.context
             .uniform_block_binding(&self.id, location, index);
         buffer.bind(index);
         self.context.unbind_buffer(consts::UNIFORM_BUFFER);
     }
 
-    pub fn use_attribute(&self, buffer: &VertexBuffer, attribute_name: &str) -> Result<(), Error> {
-        self.use_attribute_divisor(buffer, attribute_name, 0)?;
-        Ok(())
-    }
-
-    pub fn use_attribute_divisor(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-        divisor: usize,
-    ) -> Result<(), Error> {
+    ///
+    /// Uses the given [VertexBuffer] in this shader program and associates it with the given named variable.
+    /// Each value in the buffer is used when rendering one vertex using the [Program::draw_arrays] or [Program::draw_elements] methods.
+    /// Therefore the buffer must contain the same number of values as the number of vertices specified in those draw calls.
+    ///
+    pub fn use_attribute(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
         if buffer.count() > 0 {
             buffer.bind();
-            let loc = self.location(&attribute_name)?;
+            let loc = self.location(name)?;
             self.context.enable_vertex_attrib_array(loc);
             self.context
                 .vertex_attrib_pointer(loc, 1, buffer.data_type(), false, 0, 0);
-            self.context.vertex_attrib_divisor(loc, divisor as u32);
+            self.context.vertex_attrib_divisor(loc, 0);
             self.context.unbind_buffer(consts::ARRAY_BUFFER);
             self.context.unuse_program();
         }
         Ok(())
     }
 
-    pub fn use_attribute_vec2(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-    ) -> Result<(), Error> {
-        self.use_attribute_vec2_divisor(buffer, attribute_name, 0)?;
+    ///
+    /// Uses the given buffer data in this shader program and associates it with the given named variable.
+    /// Each value in the buffer is used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain the same number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+        if buffer.count() > 0 {
+            buffer.bind();
+            let loc = self.location(name)?;
+            self.context.enable_vertex_attrib_array(loc);
+            self.context
+                .vertex_attrib_pointer(loc, 1, buffer.data_type(), false, 0, 0);
+            self.context.vertex_attrib_divisor(loc, 1);
+            self.context.unbind_buffer(consts::ARRAY_BUFFER);
+            self.context.unuse_program();
+        }
         Ok(())
     }
 
-    pub fn use_attribute_vec2_divisor(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-        divisor: usize,
-    ) -> Result<(), Error> {
+    ///
+    /// Uses the given [VertexBuffer] in this shader program and associates it with the given named variable.
+    /// Each contiguous 2 values in the buffer are used when rendering one vertex using the [Program::draw_arrays] or [Program::draw_elements] methods.
+    /// Therefore the buffer must contain 2 times the number of values as the number of vertices specified in those draw calls.
+    ///
+    pub fn use_attribute_vec2(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
         if buffer.count() > 0 {
             buffer.bind();
-            let loc = self.location(&attribute_name)?;
+            let loc = self.location(name)?;
             self.context.enable_vertex_attrib_array(loc);
             self.context
                 .vertex_attrib_pointer(loc, 2, buffer.data_type(), false, 0, 0);
-            self.context.vertex_attrib_divisor(loc, divisor as u32);
+            self.context.vertex_attrib_divisor(loc, 0);
             self.context.unbind_buffer(consts::ARRAY_BUFFER);
             self.context.unuse_program();
         }
         Ok(())
     }
 
-    pub fn use_attribute_vec3(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-    ) -> Result<(), Error> {
-        self.use_attribute_vec3_divisor(buffer, attribute_name, 0)?;
+    ///
+    /// Uses the given buffer data in this shader program and associates it with the given named variable.
+    /// Each contiguous 2 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain 2 times the number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_vec2_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+        if buffer.count() > 0 {
+            buffer.bind();
+            let loc = self.location(name)?;
+            self.context.enable_vertex_attrib_array(loc);
+            self.context
+                .vertex_attrib_pointer(loc, 2, buffer.data_type(), false, 0, 0);
+            self.context.vertex_attrib_divisor(loc, 1);
+            self.context.unbind_buffer(consts::ARRAY_BUFFER);
+            self.context.unuse_program();
+        }
         Ok(())
     }
 
-    pub fn use_attribute_vec3_divisor(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-        divisor: usize,
-    ) -> Result<(), Error> {
+    ///
+    /// Uses the given [VertexBuffer] in this shader program and associates it with the given named variable.
+    /// Each contiguous 3 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain 3 times the number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_vec3(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
         if buffer.count() > 0 {
             buffer.bind();
-            let loc = self.location(&attribute_name)?;
+            let loc = self.location(&name)?;
             self.context.enable_vertex_attrib_array(loc);
             self.context
                 .vertex_attrib_pointer(loc, 3, buffer.data_type(), false, 0, 0);
-            self.context.vertex_attrib_divisor(loc, divisor as u32);
+            self.context.vertex_attrib_divisor(loc, 0);
             self.context.unbind_buffer(consts::ARRAY_BUFFER);
             self.context.unuse_program();
         }
         Ok(())
     }
 
-    pub fn use_attribute_vec4(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-    ) -> Result<(), Error> {
-        self.use_attribute_vec4_divisor(buffer, attribute_name, 0)?;
+    ///
+    /// Uses the given buffer data in this shader program and associates it with the given named variable.
+    /// Each contiguous 3 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain 3 times the number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_vec3_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+        if buffer.count() > 0 {
+            buffer.bind();
+            let loc = self.location(&name)?;
+            self.context.enable_vertex_attrib_array(loc);
+            self.context
+                .vertex_attrib_pointer(loc, 3, buffer.data_type(), false, 0, 0);
+            self.context.vertex_attrib_divisor(loc, 1);
+            self.context.unbind_buffer(consts::ARRAY_BUFFER);
+            self.context.unuse_program();
+        }
         Ok(())
     }
 
-    pub fn use_attribute_vec4_divisor(
-        &self,
-        buffer: &VertexBuffer,
-        attribute_name: &str,
-        divisor: usize,
-    ) -> Result<(), Error> {
+    ///
+    /// Uses the given [VertexBuffer] in this shader program and associates it with the given named variable.
+    /// Each contiguous 4 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain 4 times the number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_vec4(&self, name: &str, buffer: &VertexBuffer) -> Result<()> {
         if buffer.count() > 0 {
             buffer.bind();
-            let loc = self.location(&attribute_name)?;
+            let loc = self.location(name)?;
             self.context.enable_vertex_attrib_array(loc);
             self.context
                 .vertex_attrib_pointer(loc, 4, buffer.data_type(), false, 0, 0);
-            self.context.vertex_attrib_divisor(loc, divisor as u32);
+            self.context.vertex_attrib_divisor(loc, 0);
             self.context.unbind_buffer(consts::ARRAY_BUFFER);
             self.context.unuse_program();
         }
         Ok(())
     }
 
-    pub fn draw_arrays(
-        &self,
-        render_states: RenderStates,
-        cull: CullType,
-        viewport: Viewport,
-        count: u32,
-    ) {
+    ///
+    /// Uses the given buffer data in this shader program and associates it with the given named variable.
+    /// Each contiguous 4 values in the buffer are used when rendering one instance using the [Program::draw_arrays_instanced] or [Program::draw_elements_instanced] methods.
+    /// Therefore the buffer must contain 4 times the number of values as the number of instances specified in those draw calls.
+    ///
+    pub fn use_attribute_vec4_instanced(&self, name: &str, buffer: &InstanceBuffer) -> Result<()> {
+        if buffer.count() > 0 {
+            buffer.bind();
+            let loc = self.location(name)?;
+            self.context.enable_vertex_attrib_array(loc);
+            self.context
+                .vertex_attrib_pointer(loc, 4, buffer.data_type(), false, 0, 0);
+            self.context.vertex_attrib_divisor(loc, 1);
+            self.context.unbind_buffer(consts::ARRAY_BUFFER);
+            self.context.unuse_program();
+        }
+        Ok(())
+    }
+
+    ///
+    /// Draws `count` number of triangles with the given render states and viewport using this shader program.
+    /// Requires that all attributes and uniforms have been defined using the use_attribute and use_uniform methods.
+    /// Assumes that the data for the three vertices in a triangle is defined contiguous in each vertex buffer.
+    /// If you want to use an [ElementBuffer], see [Program::draw_elements].
+    ///
+    pub fn draw_arrays(&self, render_states: RenderStates, viewport: Viewport, count: u32) {
         Self::set_viewport(&self.context, viewport);
-        Self::set_cull(&self.context, cull);
         Self::set_states(&self.context, render_states);
         self.set_used();
         self.context.draw_arrays(consts::TRIANGLES, 0, count);
@@ -396,16 +421,18 @@ impl Program {
         self.context.unuse_program();
     }
 
+    ///
+    /// Same as [Program::draw_arrays] except it renders 'instance_count' instances of the same set of triangles.
+    /// Use the [Program::use_attribute_instanced], [Program::use_attribute_vec2_instanced], [Program::use_attribute_vec3_instanced] and [Program::use_attribute_vec4_instanced] methods to send unique data for each instance to the shader.
+    ///
     pub fn draw_arrays_instanced(
         &self,
         render_states: RenderStates,
-        cull: CullType,
         viewport: Viewport,
         count: u32,
         instance_count: u32,
     ) {
         Self::set_viewport(&self.context, viewport);
-        Self::set_cull(&self.context, cull);
         Self::set_states(&self.context, render_states);
         self.set_used();
         self.context
@@ -417,16 +444,19 @@ impl Program {
         self.context.unuse_program();
     }
 
+    ///
+    /// Draws the triangles defined by the given [ElementBuffer] with the given render states and viewport using this shader program.
+    /// Requires that all attributes and uniforms have been defined using the use_attribute and use_uniform methods.
+    /// If you do not want to use an [ElementBuffer], see [Program::draw_arrays]. If you only want to draw a subset of the triangles in the given [ElementBuffer], see [Program::draw_subset_of_elements].
+    ///
     pub fn draw_elements(
         &self,
         render_states: RenderStates,
-        cull: CullType,
         viewport: Viewport,
         element_buffer: &ElementBuffer,
     ) {
         self.draw_subset_of_elements(
             render_states,
-            cull,
             viewport,
             element_buffer,
             0,
@@ -434,22 +464,25 @@ impl Program {
         );
     }
 
+    ///
+    /// Draws a subset of the triangles defined by the given [ElementBuffer] with the given render states and viewport using this shader program.
+    /// Requires that all attributes and uniforms have been defined using the use_attribute and use_uniform methods.
+    /// If you do not want to use an [ElementBuffer], see [Program::draw_arrays].
+    ///
     pub fn draw_subset_of_elements(
         &self,
         render_states: RenderStates,
-        cull: CullType,
         viewport: Viewport,
         element_buffer: &ElementBuffer,
         first: u32,
         count: u32,
     ) {
         Self::set_viewport(&self.context, viewport);
-        Self::set_cull(&self.context, cull);
         Self::set_states(&self.context, render_states);
         self.set_used();
         element_buffer.bind();
         self.context
-            .draw_elements(consts::TRIANGLES, count, consts::UNSIGNED_INT, first);
+            .draw_elements(consts::TRIANGLES, count, element_buffer.data_type(), first);
         self.context.unbind_buffer(consts::ELEMENT_ARRAY_BUFFER);
 
         for location in self.vertex_attributes.values() {
@@ -458,23 +491,25 @@ impl Program {
         self.context.unuse_program();
     }
 
+    ///
+    /// Same as [Program::draw_elements] except it renders 'instance_count' instances of the same set of triangles.
+    /// Use the [Program::use_attribute_instanced], [Program::use_attribute_vec2_instanced], [Program::use_attribute_vec3_instanced] and [Program::use_attribute_vec4_instanced] methods to send unique data for each instance to the shader.
+    ///
     pub fn draw_elements_instanced(
         &self,
         render_states: RenderStates,
-        cull: CullType,
         viewport: Viewport,
         element_buffer: &ElementBuffer,
         count: u32,
     ) {
         Self::set_viewport(&self.context, viewport);
-        Self::set_cull(&self.context, cull);
         Self::set_states(&self.context, render_states);
         self.set_used();
         element_buffer.bind();
         self.context.draw_elements_instanced(
             consts::TRIANGLES,
             element_buffer.count() as u32,
-            consts::UNSIGNED_INT,
+            element_buffer.data_type(),
             0,
             count,
         );
@@ -485,17 +520,12 @@ impl Program {
         self.context.unuse_program();
     }
 
-    fn location(&self, name: &str) -> Result<u32, Error> {
+    fn location(&self, name: &str) -> Result<AttributeLocation> {
         self.set_used();
         let location = self
             .vertex_attributes
             .get(name)
-            .ok_or_else(|| ProgramError {
-                message: format!(
-                    "The attribute {} is sent to the shader but it is never used.",
-                    name
-                ),
-            })?;
+            .ok_or_else(|| CoreError::UnusedAttribute(name.to_string()))?;
         Ok(*location)
     }
 
@@ -504,7 +534,9 @@ impl Program {
     }
 
     fn set_states(context: &Context, render_states: RenderStates) {
+        Self::set_cull(context, render_states.cull);
         Self::set_write_mask(context, render_states.write_mask);
+        Self::set_clip(context, render_states.clip);
         Self::set_depth(
             context,
             Some(render_states.depth_test),
@@ -513,6 +545,27 @@ impl Program {
         Self::set_blend(context, render_states.blend);
     }
 
+    fn set_clip(context: &Context, clip: Clip) {
+        unsafe {
+            static mut CURRENT: Clip = Clip::Disabled;
+            if clip != CURRENT {
+                if let Clip::Enabled {
+                    x,
+                    y,
+                    width,
+                    height,
+                } = clip
+                {
+                    context.enable(consts::SCISSOR_TEST);
+                    context.scissor(x as i32, y as i32, width as i32, height as i32);
+                } else {
+                    context.disable(consts::SCISSOR_TEST);
+                }
+                CURRENT = clip;
+            }
+        }
+    }
+
     fn set_viewport(context: &Context, viewport: Viewport) {
         unsafe {
             static mut CURRENT_VIEWPORT: Viewport = Viewport {
@@ -522,29 +575,34 @@ impl Program {
                 height: 0,
             };
             if viewport != CURRENT_VIEWPORT {
-                context.viewport(viewport.x, viewport.y, viewport.width, viewport.height);
+                context.viewport(
+                    viewport.x,
+                    viewport.y,
+                    viewport.width as i32,
+                    viewport.height as i32,
+                );
                 CURRENT_VIEWPORT = viewport;
             }
         }
     }
 
-    fn set_cull(context: &Context, cull: CullType) {
+    fn set_cull(context: &Context, cull: Cull) {
         unsafe {
-            static mut CURRENT_CULL: CullType = CullType::None;
+            static mut CURRENT_CULL: Cull = Cull::None;
             if cull != CURRENT_CULL {
                 match cull {
-                    CullType::None => {
+                    Cull::None => {
                         context.disable(consts::CULL_FACE);
                     }
-                    CullType::Back => {
+                    Cull::Back => {
                         context.enable(consts::CULL_FACE);
                         context.cull_face(consts::BACK);
                     }
-                    CullType::Front => {
+                    Cull::Front => {
                         context.enable(consts::CULL_FACE);
                         context.cull_face(consts::FRONT);
                     }
-                    CullType::FrontAndBack => {
+                    Cull::FrontAndBack => {
                         context.enable(consts::CULL_FACE);
                         context.cull_face(consts::FRONT_AND_BACK);
                     }
@@ -554,25 +612,29 @@ impl Program {
         }
     }
 
-    fn set_blend(context: &Context, blend: Option<BlendParameters>) {
+    fn set_blend(context: &Context, blend: Blend) {
         unsafe {
-            static mut CURRENT: Option<BlendParameters> = None;
+            static mut CURRENT: Blend = Blend::Disabled;
             if blend != CURRENT {
-                if let Some(blend_parameters) = blend {
+                if let Blend::Enabled {
+                    source_rgb_multiplier,
+                    source_alpha_multiplier,
+                    destination_rgb_multiplier,
+                    destination_alpha_multiplier,
+                    rgb_equation,
+                    alpha_equation,
+                } = blend
+                {
                     context.enable(consts::BLEND);
                     context.blend_func_separate(
-                        Self::blend_const_from_multiplier(blend_parameters.source_rgb_multiplier),
-                        Self::blend_const_from_multiplier(
-                            blend_parameters.destination_rgb_multiplier,
-                        ),
-                        Self::blend_const_from_multiplier(blend_parameters.source_alpha_multiplier),
-                        Self::blend_const_from_multiplier(
-                            blend_parameters.destination_alpha_multiplier,
-                        ),
+                        Self::blend_const_from_multiplier(source_rgb_multiplier),
+                        Self::blend_const_from_multiplier(destination_rgb_multiplier),
+                        Self::blend_const_from_multiplier(source_alpha_multiplier),
+                        Self::blend_const_from_multiplier(destination_alpha_multiplier),
                     );
                     context.blend_equation_separate(
-                        Self::blend_const_from_equation(blend_parameters.rgb_equation),
-                        Self::blend_const_from_equation(blend_parameters.alpha_equation),
+                        Self::blend_const_from_equation(rgb_equation),
+                        Self::blend_const_from_equation(alpha_equation),
                     );
                 } else {
                     context.disable(consts::BLEND);
@@ -624,13 +686,13 @@ impl Program {
         }
     }
 
-    fn set_depth(context: &Context, depth_test: Option<DepthTestType>, depth_mask: bool) {
+    fn set_depth(context: &Context, depth_test: Option<DepthTest>, depth_mask: bool) {
         unsafe {
             static mut CURRENT_DEPTH_ENABLE: bool = false;
             static mut CURRENT_DEPTH_MASK: bool = true;
-            static mut CURRENT_DEPTH_TEST: DepthTestType = DepthTestType::Less;
+            static mut CURRENT_DEPTH_TEST: DepthTest = DepthTest::Less;
 
-            if depth_mask == false && depth_test == Some(DepthTestType::Always) {
+            if depth_mask == false && depth_test == Some(DepthTest::Always) {
                 if CURRENT_DEPTH_ENABLE {
                     context.disable(consts::DEPTH_TEST);
                     CURRENT_DEPTH_ENABLE = false;
@@ -650,28 +712,28 @@ impl Program {
 
             if depth_test.is_some() && depth_test.unwrap() != CURRENT_DEPTH_TEST {
                 match depth_test.unwrap() {
-                    DepthTestType::Never => {
+                    DepthTest::Never => {
                         context.depth_func(consts::NEVER);
                     }
-                    DepthTestType::Less => {
+                    DepthTest::Less => {
                         context.depth_func(consts::LESS);
                     }
-                    DepthTestType::Equal => {
+                    DepthTest::Equal => {
                         context.depth_func(consts::EQUAL);
                     }
-                    DepthTestType::LessOrEqual => {
+                    DepthTest::LessOrEqual => {
                         context.depth_func(consts::LEQUAL);
                     }
-                    DepthTestType::Greater => {
+                    DepthTest::Greater => {
                         context.depth_func(consts::GREATER);
                     }
-                    DepthTestType::NotEqual => {
+                    DepthTest::NotEqual => {
                         context.depth_func(consts::NOTEQUAL);
                     }
-                    DepthTestType::GreaterOrEqual => {
+                    DepthTest::GreaterOrEqual => {
                         context.depth_func(consts::GEQUAL);
                     }
-                    DepthTestType::Always => {
+                    DepthTest::Always => {
                         context.depth_func(consts::ALWAYS);
                     }
                 }
