@@ -1,12 +1,12 @@
-use crate::context::{consts, Context};
+//!
+//! Functionality for rendering to the screen or into textures.
+//!
+use crate::context::{consts, Context, DataType};
 use crate::core::*;
-use crate::definition::*;
-use crate::math::*;
-use crate::ImageEffect;
 
 ///
 /// Defines which channels (red, green, blue, alpha and depth) to clear when starting to write to a
-/// [render target](crate::RenderTarget) or the [screen](crate::Screen).
+/// [RenderTarget] or the [Screen].
 /// If `None` then the channel is not cleared and if `Some(value)` the channel is cleared to that value (the value must be between 0 and 1).
 ///
 #[derive(Debug, Copy, Clone, PartialEq)]
@@ -90,15 +90,15 @@ pub struct Screen {}
 
 impl Screen {
     ///
-    /// Call this function and make a render call (for example on some [object](crate::object))
+    /// Call this function and make a render call (for example one of the draw functions on [Program])
     /// in the `render` closure to render something to the screen.
     /// Before writing, the screen is cleared based on the given clear state.
     ///
-    pub fn write<F: FnOnce() -> Result<(), Error>>(
+    pub fn write<F: FnOnce() -> Result<()>>(
         context: &Context,
         clear_state: ClearState,
         render: F,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         context.bind_framebuffer(consts::DRAW_FRAMEBUFFER, None);
         clear(context, &clear_state);
         render()?;
@@ -108,16 +108,16 @@ impl Screen {
     ///
     /// Returns the RGBA color values from the screen as a list of bytes (one byte for each color channel).
     ///
-    pub fn read_color(context: &Context, viewport: Viewport) -> Result<Vec<u8>, Error> {
-        let mut pixels = vec![0u8; viewport.width * viewport.height * 4];
+    pub fn read_color(context: &Context, viewport: Viewport) -> Result<Vec<u8>> {
+        let mut pixels = vec![0u8; viewport.width as usize * viewport.height as usize * 4];
         context.bind_framebuffer(consts::READ_FRAMEBUFFER, None);
         context.read_pixels_with_u8_data(
             viewport.x as u32,
             viewport.y as u32,
-            viewport.width as u32,
-            viewport.height as u32,
+            viewport.width,
+            viewport.height,
             consts::RGBA,
-            consts::UNSIGNED_BYTE,
+            DataType::UnsignedByte,
             &mut pixels,
         );
         Ok(pixels)
@@ -128,16 +128,16 @@ impl Screen {
     /// Only available on desktop.
     ///
     #[cfg(not(target_arch = "wasm32"))]
-    pub fn read_depth(context: &Context, viewport: Viewport) -> Result<Vec<f32>, Error> {
-        let mut pixels = vec![0f32; viewport.width * viewport.height];
+    pub fn read_depth(context: &Context, viewport: Viewport) -> Result<Vec<f32>> {
+        let mut pixels = vec![0f32; viewport.width as usize * viewport.height as usize];
         context.bind_framebuffer(consts::READ_FRAMEBUFFER, None);
         context.read_pixels_with_f32_data(
             viewport.x as u32,
             viewport.y as u32,
-            viewport.width as u32,
-            viewport.height as u32,
+            viewport.width,
+            viewport.height,
             consts::DEPTH_COMPONENT,
-            consts::FLOAT,
+            DataType::Float,
             &mut pixels,
         );
         Ok(pixels)
@@ -147,39 +147,39 @@ impl Screen {
 ///
 /// The destination of applying a copy.
 ///
-pub enum CopyDestination<'a, 'b, 'c, 'd, T: TextureValueType> {
-    /// Copies to the [Screen](crate::Screen).
+pub enum CopyDestination<'a, 'b, 'c, 'd, T: TextureDataType> {
+    /// Copies to the [Screen].
     Screen,
-    /// Copies to a [ColorTargetTexture2D](crate::ColorTargetTexture2D).
+    /// Copies to a [ColorTargetTexture2D].
     ColorTexture(&'d ColorTargetTexture2D<T>),
-    /// Copies to a [DepthTargetTexture2D](crate::DepthTargetTexture2D).
+    /// Copies to a [DepthTargetTexture2D].
     DepthTexture(&'d DepthTargetTexture2D),
-    /// Copies to a [RenderTarget](crate::RenderTarget).
+    /// Copies to a [RenderTarget].
     RenderTarget(&'c RenderTarget<'a, 'b, T>),
 }
 
 ///
-/// Adds additional functionality to write to and copy from both a [ColorTargetTexture2D](crate::ColorTargetTexture2D) and
-/// a [DepthTargetTexture2D](crate::DepthTargetTexture2D) at the same time.
+/// Adds additional functionality to write to and copy from both a [ColorTargetTexture2D] and
+/// a [DepthTargetTexture2D] at the same time.
 /// It purely adds functionality, so it can be created each time it is needed, the data is saved in the textures.
 ///
-pub struct RenderTarget<'a, 'b, T: TextureValueType> {
+pub struct RenderTarget<'a, 'b, T: TextureDataType> {
     context: Context,
     id: crate::context::Framebuffer,
     color_texture: Option<&'a ColorTargetTexture2D<T>>,
     depth_texture: Option<&'b DepthTargetTexture2D>,
 }
 
-impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
+impl<'a, 'b, T: TextureDataType> RenderTarget<'a, 'b, T> {
     ///
     /// Constructs a new render target that enables rendering into the given
-    /// [color](crate::ColorTargetTexture2D) and [depth](DepthTargetTexture2D) textures.
+    /// [ColorTargetTexture2D] and [DepthTargetTexture2D] textures.
     ///
     pub fn new(
         context: &Context,
         color_texture: &'a ColorTargetTexture2D<T>,
         depth_texture: &'b DepthTargetTexture2D,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -192,11 +192,11 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
     /// Renders whatever rendered in the `render` closure into the textures defined at construction.
     /// Before writing, the textures are cleared based on the given clear state.
     ///
-    pub fn write<F: FnOnce() -> Result<(), Error>>(
+    pub fn write<F: FnOnce() -> Result<()>>(
         &self,
         clear_state: ClearState,
         render: F,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         self.bind(consts::DRAW_FRAMEBUFFER)?;
         clear(
             &self.context,
@@ -216,7 +216,7 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
     }
 
     ///
-    /// Copies the content of the color and depth textures in this render target to the specified viewport of the specified [destination](crate::CopyDestination).
+    /// Copies the content of the color and depth textures in this render target to the specified viewport of the specified destination.
     /// Only copies the channels given by the write mask.
     ///
     pub fn copy_to(
@@ -224,18 +224,18 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
         destination: CopyDestination<T>,
         viewport: Viewport,
         write_mask: WriteMask,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         let copy = || {
             let effect = get_copy_effect(&self.context)?;
             if let Some(tex) = self.color_texture {
-                effect.use_texture(tex, "colorMap")?;
+                effect.use_texture("colorMap", tex)?;
             }
             if let Some(tex) = self.depth_texture {
-                effect.use_texture(tex, "depthMap")?;
+                effect.use_texture("depthMap", tex)?;
             }
             effect.apply(
                 RenderStates {
-                    depth_test: DepthTestType::Always,
+                    depth_test: DepthTest::Always,
                     write_mask,
                     ..Default::default()
                 },
@@ -252,17 +252,19 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
             }
             CopyDestination::ColorTexture(tex) => {
                 if self.color_texture.is_none() {
-                    Err(Error::RenderTargetError {
-                        message: "Cannot copy color from a depth texture.".to_owned(),
-                    })?;
+                    Err(CoreError::RenderTargetCopy(
+                        "color".to_string(),
+                        "depth".to_string(),
+                    ))?;
                 }
                 tex.write(ClearState::none(), copy)?;
             }
             CopyDestination::DepthTexture(tex) => {
                 if self.depth_texture.is_none() {
-                    Err(Error::RenderTargetError {
-                        message: "Cannot copy depth from a color texture.".to_owned(),
-                    })?;
+                    Err(CoreError::RenderTargetCopy(
+                        "depth".to_string(),
+                        "color".to_string(),
+                    ))?;
                 }
                 tex.write(None, copy)?;
             }
@@ -273,7 +275,7 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
     pub(super) fn new_color(
         context: &Context,
         color_texture: &'a ColorTargetTexture2D<T>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -285,7 +287,7 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
     pub(super) fn new_depth(
         context: &Context,
         depth_texture: &'b DepthTargetTexture2D,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -294,7 +296,7 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
         })
     }
 
-    pub(super) fn bind(&self, target: u32) -> Result<(), Error> {
+    pub(super) fn bind(&self, target: u32) -> Result<()> {
         self.context.bind_framebuffer(target, Some(&self.id));
         if let Some(tex) = self.color_texture {
             self.context.draw_buffers(&[consts::COLOR_ATTACHMENT0]);
@@ -309,34 +311,34 @@ impl<'a, 'b, T: TextureValueType> RenderTarget<'a, 'b, T> {
     }
 }
 
-impl<T: TextureValueType> Drop for RenderTarget<'_, '_, T> {
+impl<T: TextureDataType> Drop for RenderTarget<'_, '_, T> {
     fn drop(&mut self) {
         self.context.delete_framebuffer(Some(&self.id));
     }
 }
 
 ///
-/// Adds additional functionality to write to and copy from both a [ColorTargetTexture2DArray](crate::ColorTargetTexture2DArray) and
-/// a [DepthTargetTexture2DArray](crate::DepthTargetTexture2DArray) at the same time.
+/// Adds additional functionality to write to and copy from both a [ColorTargetTexture2DArray]and
+/// a [DepthTargetTexture2DArray] at the same time.
 /// It purely adds functionality, so it can be created each time it is needed, the data is saved in the textures.
 ///
-pub struct RenderTargetArray<'a, 'b, T: TextureValueType> {
+pub struct RenderTargetArray<'a, 'b, T: TextureDataType> {
     context: Context,
     id: crate::context::Framebuffer,
     color_texture: Option<&'a ColorTargetTexture2DArray<T>>,
     depth_texture: Option<&'b DepthTargetTexture2DArray>,
 }
 
-impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
+impl<'a, 'b, T: TextureDataType> RenderTargetArray<'a, 'b, T> {
     ///
     /// Constructs a new render target array that enables rendering into the given
-    /// [color](crate::ColorTargetTexture2DArray) and [depth](DepthTargetTexture2DArray) array textures.
+    /// [ColorTargetTexture2DArray] and [DepthTargetTexture2DArray] array textures.
     ///
     pub fn new(
         context: &Context,
         color_texture: &'a ColorTargetTexture2DArray<T>,
         depth_texture: &'b DepthTargetTexture2DArray,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -348,7 +350,7 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
     pub(super) fn new_color(
         context: &Context,
         color_texture: &'a ColorTargetTexture2DArray<T>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -360,7 +362,7 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
     pub(super) fn new_depth(
         context: &Context,
         depth_texture: &'b DepthTargetTexture2DArray,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Ok(Self {
             context: context.clone(),
             id: new_framebuffer(context)?,
@@ -376,13 +378,13 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
     /// The depth is written to the depth texture defined by `depth_layer`.
     /// Before writing, the textures are cleared based on the given clear state.
     ///
-    pub fn write<F: FnOnce() -> Result<(), Error>>(
+    pub fn write(
         &self,
-        color_layers: &[usize],
-        depth_layer: usize,
+        color_layers: &[u32],
+        depth_layer: u32,
         clear_state: ClearState,
-        render: F,
-    ) -> Result<(), Error> {
+        render: impl FnOnce() -> Result<()>,
+    ) -> Result<()> {
         self.bind(Some(color_layers), Some(depth_layer))?;
         clear(
             &self.context,
@@ -402,30 +404,30 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
     }
 
     ///
-    /// Copies the content of the specified color and depth layers in this render target to the given viewport of the given [destination](crate::CopyDestination).
+    /// Copies the content of the specified color and depth layers in this render target to the given viewport of the given destination.
     /// Only copies the channels specified by the write mask.
     ///
     pub fn copy_to(
         &self,
-        color_layer: usize,
-        depth_layer: usize,
+        color_layer: u32,
+        depth_layer: u32,
         destination: CopyDestination<T>,
         viewport: Viewport,
         write_mask: WriteMask,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         let copy = || {
             let effect = get_copy_array_effect(&self.context)?;
             if let Some(tex) = self.color_texture {
-                effect.use_texture_array(tex, "colorMap")?;
+                effect.use_texture_array("colorMap", tex)?;
                 effect.use_uniform_int("colorLayer", &(color_layer as i32))?;
             }
             if let Some(tex) = self.depth_texture {
-                effect.use_texture_array(tex, "depthMap")?;
+                effect.use_texture_array("depthMap", tex)?;
                 effect.use_uniform_int("depthLayer", &(depth_layer as i32))?;
             }
             effect.apply(
                 RenderStates {
-                    depth_test: DepthTestType::Always,
+                    depth_test: DepthTest::Always,
                     write_mask,
                     ..Default::default()
                 },
@@ -442,17 +444,19 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
             }
             CopyDestination::ColorTexture(tex) => {
                 if self.color_texture.is_none() {
-                    Err(Error::RenderTargetError {
-                        message: "Cannot copy color from a depth texture.".to_owned(),
-                    })?;
+                    Err(CoreError::RenderTargetCopy(
+                        "color".to_string(),
+                        "depth".to_string(),
+                    ))?;
                 }
                 tex.write(ClearState::none(), copy)?;
             }
             CopyDestination::DepthTexture(tex) => {
                 if self.depth_texture.is_none() {
-                    Err(Error::RenderTargetError {
-                        message: "Cannot copy depth from a color texture.".to_owned(),
-                    })?;
+                    Err(CoreError::RenderTargetCopy(
+                        "depth".to_string(),
+                        "color".to_string(),
+                    ))?;
                 }
                 tex.write(None, copy)?;
             }
@@ -460,11 +464,7 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
         Ok(())
     }
 
-    fn bind(
-        &self,
-        color_layers: Option<&[usize]>,
-        depth_layer: Option<usize>,
-    ) -> Result<(), Error> {
+    fn bind(&self, color_layers: Option<&[u32]>, depth_layer: Option<u32>) -> Result<()> {
         self.context
             .bind_framebuffer(consts::DRAW_FRAMEBUFFER, Some(&self.id));
         if let Some(color_texture) = self.color_texture {
@@ -475,7 +475,7 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
                         .collect::<Vec<u32>>(),
                 );
                 for channel in 0..color_layers.len() {
-                    color_texture.bind_as_color_target(color_layers[channel], channel);
+                    color_texture.bind_as_color_target(color_layers[channel], channel as u32);
                 }
             }
         }
@@ -490,27 +490,23 @@ impl<'a, 'b, T: TextureValueType> RenderTargetArray<'a, 'b, T> {
     }
 }
 
-impl<T: TextureValueType> Drop for RenderTargetArray<'_, '_, T> {
+impl<T: TextureDataType> Drop for RenderTargetArray<'_, '_, T> {
     fn drop(&mut self) {
         self.context.delete_framebuffer(Some(&self.id));
     }
 }
 
-fn new_framebuffer(context: &Context) -> Result<crate::context::Framebuffer, Error> {
+fn new_framebuffer(context: &Context) -> Result<crate::context::Framebuffer> {
     Ok(context
         .create_framebuffer()
-        .ok_or_else(|| Error::RenderTargetError {
-            message: "Failed to create framebuffer".to_string(),
-        })?)
+        .ok_or(CoreError::RenderTargetCreation)?)
 }
 
 #[cfg(feature = "debug")]
-fn check(context: &Context) -> Result<(), Error> {
-    context.check_framebuffer_status().or_else(|status| {
-        Err(Error::RenderTargetError {
-            message: format!("Failed to create frame buffer: {}", status),
-        })
-    })
+fn check(context: &Context) -> Result<()> {
+    context
+        .check_framebuffer_status()
+        .or_else(|status| Err(CoreError::RenderTargetCreation))
 }
 
 fn clear(context: &Context, clear_state: &ClearState) {
@@ -550,7 +546,7 @@ fn clear(context: &Context, clear_state: &ClearState) {
     });
 }
 
-fn get_copy_effect(context: &Context) -> Result<&ImageEffect, Error> {
+fn get_copy_effect(context: &Context) -> Result<&ImageEffect> {
     unsafe {
         static mut COPY_EFFECT: Option<ImageEffect> = None;
         if COPY_EFFECT.is_none() {
@@ -572,7 +568,7 @@ fn get_copy_effect(context: &Context) -> Result<&ImageEffect, Error> {
     }
 }
 
-fn get_copy_array_effect(context: &Context) -> Result<&ImageEffect, Error> {
+fn get_copy_array_effect(context: &Context) -> Result<&ImageEffect> {
     unsafe {
         static mut COPY_EFFECT: Option<ImageEffect> = None;
         if COPY_EFFECT.is_none() {
