@@ -1,4 +1,6 @@
-use crate::*;
+use crate::core::*;
+use crate::window::*;
+use crate::Result;
 #[doc(hidden)]
 pub use egui;
 
@@ -8,8 +10,8 @@ pub use egui;
 pub struct GUI {
     context: Context,
     egui_context: egui::CtxRef,
-    width: usize,
-    height: usize,
+    width: u32,
+    height: u32,
     program: Program,
     texture_version: u64,
     texture: Option<Texture2D>,
@@ -19,7 +21,7 @@ impl GUI {
     ///
     /// Creates a new GUI.
     ///
-    pub fn new(context: &Context) -> Result<Self, Error> {
+    pub fn new(context: &Context) -> Result<Self> {
         Ok(GUI {
             egui_context: egui::CtxRef::default(),
             context: context.clone(),
@@ -27,7 +29,19 @@ impl GUI {
             height: 0,
             texture_version: 0,
             texture: None,
-            program: Program::from_source(context, VERTEX_SHADER_SOURCE, FRAGMENT_SHADER_SOURCE)?,
+            program: Program::from_source(
+                context,
+                &format!(
+                    "{}{}",
+                    include_str!("../core/shared.frag"),
+                    include_str!("shaders/egui.vert")
+                ),
+                &format!(
+                    "{}{}",
+                    include_str!("../core/shared.frag"),
+                    include_str!("shaders/egui.frag")
+                ),
+            )?,
         })
     }
 
@@ -40,7 +54,7 @@ impl GUI {
         &mut self,
         frame_input: &mut FrameInput,
         callback: F,
-    ) -> Result<bool, Error> {
+    ) -> Result<bool> {
         self.width = frame_input.window_width;
         self.height = frame_input.window_height;
         let input_state = construct_input_state(frame_input);
@@ -51,7 +65,12 @@ impl GUI {
         for event in frame_input.events.iter_mut() {
             if self.egui_context.wants_pointer_input() {
                 match event {
-                    Event::MouseClick {
+                    Event::MousePress {
+                        ref mut handled, ..
+                    } => {
+                        *handled = true;
+                    }
+                    Event::MouseRelease {
                         ref mut handled, ..
                     } => {
                         *handled = true;
@@ -73,7 +92,12 @@ impl GUI {
 
             if self.egui_context.wants_keyboard_input() {
                 match event {
-                    Event::Key {
+                    Event::KeyRelease {
+                        ref mut handled, ..
+                    } => {
+                        *handled = true;
+                    }
+                    Event::KeyPress {
                         ref mut handled, ..
                     } => {
                         *handled = true;
@@ -90,7 +114,7 @@ impl GUI {
     /// Render the GUI defined in the [update](Self::update) function. Must be called in a render target render function,
     /// for example in the callback function of [Screen::write](crate::Screen::write).
     ///
-    pub fn render(&mut self) -> Result<(), Error> {
+    pub fn render(&mut self) -> Result<()> {
         let (_, shapes) = self.egui_context.end_frame();
         let clipped_meshes = self.egui_context.tessellate(shapes);
 
@@ -108,9 +132,9 @@ impl GUI {
                 &self.context,
                 &CPUTexture {
                     data: pixels,
-                    format: Format::SRGBA,
-                    width: egui_texture.width,
-                    height: egui_texture.height,
+                    format: Format::RGBA,
+                    width: egui_texture.width as u32,
+                    height: egui_texture.height as u32,
                     mip_map_filter: None,
                     wrap_s: Wrapping::ClampToEdge,
                     wrap_t: Wrapping::ClampToEdge,
@@ -120,11 +144,26 @@ impl GUI {
             self.texture_version = egui_texture.version;
         };
 
-        for egui::ClippedMesh(_, mesh) in clipped_meshes {
+        let scale = self.egui_context.pixels_per_point();
+        let viewport = Viewport::new_at_origo(
+            (self.width as f32 * scale).round() as u32,
+            (self.height as f32 * scale).round() as u32,
+        );
+
+        for egui::ClippedMesh(rect, mesh) in clipped_meshes {
+            let width = rect.max.x - rect.min.x;
+            let height = rect.max.y - rect.min.y;
+            let clipping = Clip::Enabled {
+                x: (rect.min.x * scale) as u32,
+                y: ((self.height as f32 - rect.max.y) * scale) as u32,
+                width: (width * scale) as u32,
+                height: (height * scale) as u32,
+            };
             self.paint_mesh(
+                viewport,
+                clipping,
                 self.width,
                 self.height,
-                self.egui_context.pixels_per_point(),
                 &mesh,
                 self.texture.as_ref().unwrap(),
             )?;
@@ -134,12 +173,13 @@ impl GUI {
 
     fn paint_mesh(
         &self,
-        width: usize,
-        height: usize,
-        pixels_per_point: f32,
+        viewport: Viewport,
+        clip: Clip,
+        width: u32,
+        height: u32,
         mesh: &egui::paint::Mesh,
         texture: &Texture2D,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         debug_assert!(mesh.is_valid());
 
         let mut positions = Vec::new();
@@ -157,121 +197,72 @@ impl GUI {
         }
         let indices: Vec<u32> = mesh.indices.iter().map(|idx| *idx as u32).collect();
 
-        let position_buffer = VertexBuffer::new_with_static_f32(&self.context, &positions)?;
-        let uv_buffer = VertexBuffer::new_with_static_f32(&self.context, &uvs)?;
-        let color_buffer = VertexBuffer::new_with_static_f32(&self.context, &colors)?;
-        let index_buffer = ElementBuffer::new_with_u32(&self.context, &indices)?;
-
-        let viewport = Viewport::new_at_origo(
-            (width as f32 * pixels_per_point).round() as usize,
-            (height as f32 * pixels_per_point).round() as usize,
-        );
+        let position_buffer = VertexBuffer::new_with_static(&self.context, &positions)?;
+        let uv_buffer = VertexBuffer::new_with_static(&self.context, &uvs)?;
+        let color_buffer = VertexBuffer::new_with_static(&self.context, &colors)?;
+        let index_buffer = ElementBuffer::new_with(&self.context, &indices)?;
 
         let render_states = RenderStates {
-            blend: Some(BlendParameters {
+            blend: Blend::Enabled {
                 source_rgb_multiplier: BlendMultiplierType::One,
-                destination_rgb_multiplier: BlendMultiplierType::OneMinusSrcAlpha,
                 source_alpha_multiplier: BlendMultiplierType::OneMinusDstAlpha,
+                destination_rgb_multiplier: BlendMultiplierType::OneMinusSrcAlpha,
                 destination_alpha_multiplier: BlendMultiplierType::One,
-                ..Default::default()
-            }),
-            depth_test: DepthTestType::Always,
+                rgb_equation: BlendEquationType::Add,
+                alpha_equation: BlendEquationType::Add,
+            },
+            depth_test: DepthTest::Always,
+            clip,
             ..Default::default()
         };
 
-        self.program.use_texture(texture, "u_sampler")?;
+        self.program.use_texture("u_sampler", texture)?;
         self.program
             .use_uniform_vec2("u_screen_size", &vec2(width as f32, height as f32))?;
 
-        self.program.use_attribute_vec2(&position_buffer, "a_pos")?;
-        self.program.use_attribute_vec4(&color_buffer, "a_srgba")?;
-        self.program.use_attribute_vec2(&uv_buffer, "a_tc")?;
+        self.program.use_attribute_vec2("a_pos", &position_buffer)?;
+        self.program.use_attribute_vec4("a_srgba", &color_buffer)?;
+        self.program.use_attribute_vec2("a_tc", &uv_buffer)?;
 
         self.program
-            .draw_elements(render_states, CullType::None, viewport, &index_buffer);
+            .draw_elements(render_states, viewport, &index_buffer);
         Ok(())
     }
 }
 
-const VERTEX_SHADER_SOURCE: &str = r#"
-    uniform vec2 u_screen_size;
-    in vec2 a_pos;
-    in vec2 a_tc;
-    in vec4 a_srgba;
-    out vec4 v_rgba;
-    out vec2 v_tc;
-    // 0-1 linear  from  0-255 sRGB
-    vec3 linear_from_srgb(vec3 srgb) {
-        bvec3 cutoff = lessThan(srgb, vec3(10.31475));
-        vec3 lower = srgb / vec3(3294.6);
-        vec3 higher = pow((srgb + vec3(14.025)) / vec3(269.025), vec3(2.4));
-        return mix(higher, lower, vec3(cutoff));
-    }
-    vec4 linear_from_srgba(vec4 srgba) {
-        return vec4(linear_from_srgb(srgba.rgb), srgba.a / 255.0);
-    }
-    void main() {
-        gl_Position = vec4(
-            2.0 * a_pos.x / u_screen_size.x - 1.0,
-            1.0 - 2.0 * a_pos.y / u_screen_size.y,
-            0.0,
-            1.0);
-        // egui encodes vertex colors in gamma spaces, so we must decode the colors here:
-        v_rgba = linear_from_srgba(a_srgba);
-        v_tc = a_tc;
-    }
-"#;
-
-const FRAGMENT_SHADER_SOURCE: &str = r#"
-    uniform sampler2D u_sampler;
-    in vec4 v_rgba;
-    in vec2 v_tc;
-    layout (location = 0) out vec4 color;
-    // 0-255 sRGB  from  0-1 linear
-    vec3 srgb_from_linear(vec3 rgb) {
-        bvec3 cutoff = lessThan(rgb, vec3(0.0031308));
-        vec3 lower = rgb * vec3(3294.6);
-        vec3 higher = vec3(269.025) * pow(rgb, vec3(1.0 / 2.4)) - vec3(14.025);
-        return mix(higher, lower, vec3(cutoff));
-    }
-    vec4 srgba_from_linear(vec4 rgba) {
-        return vec4(srgb_from_linear(rgba.rgb), 255.0 * rgba.a);
-    }
-    void main() {
-        // The texture is set up with `SRGB8_ALPHA8`, so no need to decode here!
-        vec4 texture_rgba = texture(u_sampler, v_tc);
-        /// Multiply vertex color with texture color (in linear space).
-        color = v_rgba * texture_rgba;
-        // We must gamma-encode again since WebGL doesn't support linear blending in the framebuffer.
-        color = srgba_from_linear(color) / 255.0;
-        // WebGL doesn't support linear blending in the framebuffer,
-        // so we apply this hack to at least get a bit closer to the desired blending:
-        color.a = pow(color.a, 1.6); // Empiric nonsense
-    }
-"#;
-
 fn construct_input_state(frame_input: &mut FrameInput) -> egui::RawInput {
     let mut scroll_delta = egui::Vec2::ZERO;
     let mut egui_modifiers = egui::Modifiers::default();
     let mut egui_events = Vec::new();
     for event in frame_input.events.iter() {
         match event {
-            Event::Key {
+            Event::KeyPress {
                 kind,
-                state,
                 modifiers,
                 handled,
             } => {
                 if !handled {
                     egui_events.push(egui::Event::Key {
                         key: translate_to_egui_key_code(kind),
-                        pressed: *state == State::Pressed,
+                        pressed: true,
                         modifiers: map_modifiers(modifiers),
                     });
                 }
             }
-            Event::MouseClick {
-                state,
+            Event::KeyRelease {
+                kind,
+                modifiers,
+                handled,
+            } => {
+                if !handled {
+                    egui_events.push(egui::Event::Key {
+                        key: translate_to_egui_key_code(kind),
+                        pressed: false,
+                        modifiers: map_modifiers(modifiers),
+                    });
+                }
+            }
+            Event::MousePress {
                 button,
                 position,
                 modifiers,
@@ -288,7 +279,29 @@ fn construct_input_state(frame_input: &mut FrameInput) -> egui::RawInput {
                             MouseButton::Right => egui::PointerButton::Secondary,
                             MouseButton::Middle => egui::PointerButton::Middle,
                         },
-                        pressed: *state == State::Pressed,
+                        pressed: true,
+                        modifiers: map_modifiers(modifiers),
+                    });
+                }
+            }
+            Event::MouseRelease {
+                button,
+                position,
+                modifiers,
+                handled,
+            } => {
+                if !handled {
+                    egui_events.push(egui::Event::PointerButton {
+                        pos: egui::Pos2 {
+                            x: position.0 as f32,
+                            y: position.1 as f32,
+                        },
+                        button: match button {
+                            MouseButton::Left => egui::PointerButton::Primary,
+                            MouseButton::Right => egui::PointerButton::Secondary,
+                            MouseButton::Middle => egui::PointerButton::Middle,
+                        },
+                        pressed: false,
                         modifiers: map_modifiers(modifiers),
                     });
                 }
@@ -314,15 +327,7 @@ fn construct_input_state(frame_input: &mut FrameInput) -> egui::RawInput {
                     scroll_delta = egui::Vec2::new(delta.0 as f32, delta.1 as f32);
                 }
             }
-            Event::ModifiersChange { modifiers } => {
-                egui_modifiers = egui::Modifiers {
-                    alt: modifiers.alt == State::Pressed,
-                    ctrl: modifiers.ctrl == State::Pressed,
-                    shift: modifiers.shift == State::Pressed,
-                    mac_cmd: modifiers.command == State::Pressed,
-                    command: modifiers.command == State::Pressed,
-                }
-            }
+            Event::ModifiersChange { modifiers } => egui_modifiers = map_modifiers(modifiers),
             _ => (),
         }
     }
@@ -409,10 +414,10 @@ fn translate_to_egui_key_code(key: &crate::Key) -> egui::Key {
 
 fn map_modifiers(modifiers: &Modifiers) -> egui::Modifiers {
     egui::Modifiers {
-        alt: modifiers.alt == State::Pressed,
-        ctrl: modifiers.ctrl == State::Pressed,
-        shift: modifiers.shift == State::Pressed,
-        command: modifiers.command == State::Pressed,
-        mac_cmd: cfg!(target_os = "macos") && modifiers.command == State::Pressed,
+        alt: modifiers.alt,
+        ctrl: modifiers.ctrl,
+        shift: modifiers.shift,
+        command: modifiers.command,
+        mac_cmd: cfg!(target_os = "macos") && modifiers.command,
     }
 }
