@@ -1,6 +1,7 @@
-use crate::frame::*;
-use crate::window::WindowSettings;
-use crate::Context;
+use crate::context::Context;
+use crate::core::Viewport;
+use crate::window::*;
+use crate::Result;
 use serde::Serialize;
 use std::cell::RefCell;
 use std::rc::Rc;
@@ -8,13 +9,32 @@ use wasm_bindgen::prelude::*;
 use wasm_bindgen::JsCast;
 use web_sys::WebGl2RenderingContext;
 
-#[derive(Debug)]
-pub enum WindowError {
-    WindowCreationError { message: String },
-    ContextError { message: String },
-    PerformanceError { message: String },
-    EventListenerError { message: String },
-    CanvasError { message: String },
+use thiserror::Error;
+
+///
+/// Error related to the canvas.
+///
+#[derive(Error, Debug)]
+#[allow(missing_docs)]
+pub enum CanvasError {
+    #[error("failed creating a new window")]
+    WindowCreation,
+    #[error("unable to get document from canvas")]
+    DocumentMissing,
+    #[error("unable to get canvas")]
+    CanvasMissing,
+    #[error("unable to convert canvas to html canvas: {0}")]
+    CanvasConvertFailed(String),
+    #[error("unable to get webgl2 context for the given canvas, maybe the browser doesn't support WebGL2{0}")]
+    WebGL2NotSupported(String),
+    #[error("unable to get EXT_color_buffer_float extension for the given canvas, maybe the browser doesn't support EXT_color_buffer_float: {0}")]
+    ColorBufferFloatNotSupported(String),
+    #[error("unable to get OES_texture_float extension for the given canvas, maybe the browser doesn't support OES_texture_float: {0}")]
+    OESTextureFloatNotSupported(String),
+    #[error("performance (for timing) is not found on the window")]
+    PerformanceMissing,
+    #[error("unable to add {0} event listener: {1}")]
+    EventListenerFail(String, String),
 }
 
 pub struct Window {
@@ -30,15 +50,11 @@ pub struct Window {
 }
 
 impl Window {
-    pub fn new(settings: WindowSettings) -> Result<Window, WindowError> {
-        let websys_window = web_sys::window().ok_or(WindowError::WindowCreationError {
-            message: "Unable to create web window".to_string(),
-        })?;
+    pub fn new(settings: WindowSettings) -> Result<Window> {
+        let websys_window = web_sys::window().ok_or(CanvasError::WindowCreation)?;
         let document = websys_window
             .document()
-            .ok_or(WindowError::WindowCreationError {
-                message: "Unable to get document".to_string(),
-            })?;
+            .ok_or(CanvasError::DocumentMissing)?;
 
         let mut window = Window {
             canvas: None,
@@ -55,12 +71,7 @@ impl Window {
             window.set_canvas(
                 canvas
                     .dyn_into::<web_sys::HtmlCanvasElement>()
-                    .map_err(|e| WindowError::WindowCreationError {
-                        message: format!(
-                            "Unable to convert to HtmlCanvasElement. Error code: {:?}",
-                            e
-                        ),
-                    })?,
+                    .map_err(|e| CanvasError::CanvasConvertFailed(format!("{:?}", e)))?,
             )?;
         };
         Ok(window)
@@ -70,57 +81,64 @@ impl Window {
     /// Get the canvas which is rendered to when using [Screen](crate::Screen).
     /// If there is no canvas specified using the set_canvas function and no default canvas is found, this will return an error.
     ///
-    pub fn canvas(&self) -> Result<&web_sys::HtmlCanvasElement, WindowError> {
-        self.canvas.as_ref().ok_or(WindowError::CanvasError {
-            message: "Could not find a canvas.".to_string(),
-        })
+    pub fn canvas(&self) -> Result<&web_sys::HtmlCanvasElement> {
+        self.canvas
+            .as_ref()
+            .ok_or(Box::new(CanvasError::CanvasMissing))
     }
 
     ///
     /// Specifies the canvas to write to when using [Screen](crate::Screen). Will overwrite the default canvas if any has been found.
     ///
-    pub fn set_canvas(&mut self, canvas: web_sys::HtmlCanvasElement) -> Result<(), WindowError> {
+    pub fn set_canvas(&mut self, canvas: web_sys::HtmlCanvasElement) -> Result<()> {
         self.canvas = Some(canvas);
         self.set_canvas_size()?;
         Ok(())
     }
 
-    pub fn size(&self) -> Result<(usize, usize), WindowError> {
-        let canvas = self.canvas.as_ref().ok_or(WindowError::CanvasError {
-            message: "Could not find a canvas.".to_string(),
-        })?;
-        Ok((canvas.width() as usize, canvas.height() as usize))
+    pub fn size(&self) -> Result<(u32, u32)> {
+        let canvas = self.canvas.as_ref().ok_or(CanvasError::CanvasMissing)?;
+        Ok((canvas.width(), canvas.height()))
     }
 
-    pub fn viewport(&self) -> Result<crate::Viewport, WindowError> {
+    pub fn viewport(&self) -> Result<Viewport> {
         let (w, h) = self.size()?;
-        Ok(crate::Viewport::new_at_origo(w, h))
+        Ok(Viewport::new_at_origo(w, h))
     }
 
-    pub fn gl(&self) -> Result<crate::Context, WindowError> {
+    pub fn gl(&self) -> Result<Context> {
         let context_options = ContextOptions {
             antialias: self.settings.multisamples > 0,
         };
-        let context = self.canvas.as_ref().ok_or(WindowError::CanvasError {message: "Could not find a canvas.".to_string()})?
-            .get_context_with_context_options("webgl2", &JsValue::from_serde(&context_options).unwrap())
-            .map_err(|e| WindowError::ContextError {message: format!("Unable to get webgl2 context for the given canvas. Maybe your browser doesn't support WebGL2? Error code: {:?}", e)})?
-            .ok_or(WindowError::ContextError {message: "Unable to get webgl2 context for the given canvas. Maybe your browser doesn't support WebGL2?".to_string()})?
-            .dyn_into::<WebGl2RenderingContext>().map_err(|e| WindowError::ContextError {message: format!("Unable to get webgl2 context for the given canvas. Maybe your browser doesn't support WebGL2? Error code: {:?}", e)})?;
-        context.get_extension("EXT_color_buffer_float").map_err(|e| WindowError::ContextError {message: format!("Unable to get EXT_color_buffer_float extension for the given context. Maybe your browser doesn't support the get color_buffer_float extension? Error code: {:?}", e)})?;
-        context.get_extension("OES_texture_float").map_err(|e| WindowError::ContextError {message: format!("Unable to get OES_texture_float extension for the given context. Maybe your browser doesn't support the get OES_texture_float extension? Error code: {:?}", e)})?;
+        let context = self
+            .canvas
+            .as_ref()
+            .ok_or(CanvasError::CanvasMissing)?
+            .get_context_with_context_options(
+                "webgl2",
+                &JsValue::from_serde(&context_options).unwrap(),
+            )
+            .map_err(|e| CanvasError::WebGL2NotSupported(format!(": {:?}", e)))?
+            .ok_or(CanvasError::WebGL2NotSupported("".to_string()))?
+            .dyn_into::<WebGl2RenderingContext>()
+            .map_err(|e| CanvasError::WebGL2NotSupported(format!(": {:?}", e)))?;
+        context
+            .get_extension("EXT_color_buffer_float")
+            .map_err(|e| CanvasError::ColorBufferFloatNotSupported(format!("{:?}", e)))?;
+        context
+            .get_extension("OES_texture_float")
+            .map_err(|e| CanvasError::OESTextureFloatNotSupported(format!(": {:?}", e)))?;
         Ok(Context::new(context))
     }
 
-    pub fn render_loop<F: 'static>(mut self, mut callback: F) -> Result<(), WindowError>
+    pub fn render_loop<F: 'static>(mut self, mut callback: F) -> Result<()>
     where
         F: FnMut(FrameInput) -> FrameOutput,
     {
         let performance = self
             .window
             .performance()
-            .ok_or(WindowError::PerformanceError {
-                message: "Performance (for timing) is not found on the window.".to_string(),
-            })?;
+            .ok_or(CanvasError::PerformanceMissing)?;
         let mut last_time = performance.now();
         let mut accumulated_time = 0.0;
         let mut first_frame = true;
@@ -151,16 +169,16 @@ impl Window {
             let device_pixel_ratio = self.pixels_per_point();
             let canvas = self.canvas.as_ref().unwrap();
             let (width, height) = (
-                (canvas.width() as f64 / device_pixel_ratio) as usize,
-                (canvas.height() as f64 / device_pixel_ratio) as usize,
+                (canvas.width() as f64 / device_pixel_ratio) as u32,
+                (canvas.height() as f64 / device_pixel_ratio) as u32,
             );
-            let frame_input = crate::FrameInput {
+            let frame_input = FrameInput {
                 events,
                 elapsed_time,
                 accumulated_time,
-                viewport: crate::Viewport::new_at_origo(
-                    (device_pixel_ratio * width as f64) as usize,
-                    (device_pixel_ratio * height as f64) as usize,
+                viewport: Viewport::new_at_origo(
+                    (device_pixel_ratio * width as f64) as u32,
+                    (device_pixel_ratio * height as f64) as u32,
                 ),
                 window_width: width,
                 window_height: height,
@@ -188,10 +206,8 @@ impl Window {
         }
     }
 
-    fn set_canvas_size(&self) -> Result<(), WindowError> {
-        let canvas = self.canvas.as_ref().ok_or(WindowError::CanvasError {
-            message: "Could not find a canvas.".to_string(),
-        })?;
+    fn set_canvas_size(&self) -> Result<()> {
+        let canvas = self.canvas.as_ref().ok_or(CanvasError::CanvasMissing)?;
         let (window_width, window_height) = (
             self.window.inner_width().unwrap().as_f64().unwrap() as u32,
             self.window.inner_height().unwrap().as_f64().unwrap() as u32,
@@ -238,7 +254,7 @@ impl Window {
         Ok(())
     }
 
-    fn add_context_menu_event_listener(&mut self) -> Result<(), WindowError> {
+    fn add_context_menu_event_listener(&mut self) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::Event| {
             event.prevent_default();
             event.stop_propagation();
@@ -249,26 +265,24 @@ impl Window {
         Ok(())
     }
 
-    fn add_resize_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<(), WindowError> {
+    fn add_resize_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move || {
             input.borrow_mut().request_animation_frame();
         }) as Box<dyn FnMut()>);
         self.canvas()?
             .add_event_listener_with_callback("resize", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!("Unable to add resize event listener. Error code: {:?}", e),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("resize".to_string(), format!("{:?}", e))
             })?;
         self.closures.push(closure);
         Ok(())
     }
 
-    fn add_mouseleave_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_mouseleave_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::MouseEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
+                input.mouse_pressed = None;
                 input.events.push(Event::MouseLeave);
                 event.stop_propagation();
                 event.prevent_default();
@@ -278,20 +292,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("mouseleave", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add mouse leave event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("mouseleave".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_mouseevent.push(closure);
         Ok(())
     }
 
-    fn add_mouseenter_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_mouseenter_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::MouseEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -304,20 +312,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("mouseenter", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add mouse enter event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("mouseenter".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_mouseevent.push(closure);
         Ok(())
     }
 
-    fn add_mousedown_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_mousedown_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::MouseEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -329,8 +331,8 @@ impl Window {
                 };
                 if let Some(button) = button {
                     let modifiers = input.modifiers;
-                    input.events.push(Event::MouseClick {
-                        state: State::Pressed,
+                    input.mouse_pressed = Some(button);
+                    input.events.push(Event::MousePress {
                         button,
                         position: (event.offset_x() as f64, event.offset_y() as f64),
                         modifiers,
@@ -345,17 +347,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("mousedown", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add mouse down event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("mousedown".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_mouseevent.push(closure);
         Ok(())
     }
 
-    fn add_mouseup_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<(), WindowError> {
+    fn add_mouseup_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::MouseEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -367,8 +366,8 @@ impl Window {
                 };
                 if let Some(button) = button {
                     let modifiers = input.modifiers;
-                    input.events.push(Event::MouseClick {
-                        state: State::Released,
+                    input.mouse_pressed = None;
+                    input.events.push(Event::MouseRelease {
                         button,
                         position: (event.offset_x() as f64, event.offset_y() as f64),
                         modifiers,
@@ -383,17 +382,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("mouseup", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!("Unable to add mouse up event listener. Error code: {:?}", e),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("mouseup".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_mouseevent.push(closure);
         Ok(())
     }
 
-    fn add_mousemove_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_mousemove_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::MouseEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -403,7 +399,9 @@ impl Window {
                     (0.0, 0.0)
                 };
                 let modifiers = input.modifiers;
+                let button = input.mouse_pressed;
                 input.events.push(Event::MouseMotion {
+                    button,
                     delta,
                     position: (event.offset_x() as f64, event.offset_y() as f64),
                     modifiers,
@@ -418,20 +416,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("mousemove", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add mouse move event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("mousemove".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_mouseevent.push(closure);
         Ok(())
     }
 
-    fn add_mousewheel_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_mousewheel_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::WheelEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -449,25 +441,20 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("wheel", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!("Unable to add wheel event listener. Error code: {:?}", e),
-            })?;
+            .map_err(|e| CanvasError::EventListenerFail("wheel".to_string(), format!("{:?}", e)))?;
         self.closures_with_wheelevent.push(closure);
         Ok(())
     }
 
-    fn add_touchstart_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_touchstart_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::TouchEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
                 if event.touches().length() == 1 {
                     let touch = event.touches().item(0).unwrap();
                     let modifiers = input.modifiers;
-                    input.events.push(Event::MouseClick {
-                        state: State::Pressed,
+                    input.mouse_pressed = Some(MouseButton::Left);
+                    input.events.push(Event::MousePress {
                         button: MouseButton::Left,
                         position: (touch.page_x() as f64, touch.page_y() as f64),
                         modifiers,
@@ -496,27 +483,21 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("touchstart", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add touch start event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("touchstart".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_touchevent.push(closure);
         Ok(())
     }
 
-    fn add_touchend_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_touchend_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::TouchEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
                 if let Some((x, y)) = input.last_position {
                     let modifiers = input.modifiers;
-                    input.events.push(Event::MouseClick {
-                        state: State::Released,
+                    input.mouse_pressed = None;
+                    input.events.push(Event::MouseRelease {
                         button: MouseButton::Left,
                         position: (x as f64, y as f64),
                         modifiers,
@@ -533,20 +514,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("touchend", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add touch end event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("touchend".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_touchevent.push(closure);
         Ok(())
     }
 
-    fn add_touchmove_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_touchmove_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::TouchEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -554,7 +529,9 @@ impl Window {
                     let touch = event.touches().item(0).unwrap();
                     if let Some((x, y)) = input.last_position {
                         let modifiers = input.modifiers;
+                        let button = input.mouse_pressed;
                         input.events.push(Event::MouseMotion {
+                            button,
                             delta: ((touch.page_x() - x) as f64, (touch.page_y() - y) as f64),
                             position: (touch.page_x() as f64, touch.page_y() as f64),
                             modifiers,
@@ -596,20 +573,14 @@ impl Window {
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
             .add_event_listener_with_callback("touchmove", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!(
-                    "Unable to add touch move event listener. Error code: {:?}",
-                    e
-                ),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("touchmove".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_touchevent.push(closure);
         Ok(())
     }
 
-    fn add_key_down_event_listener(
-        &mut self,
-        input: Rc<RefCell<Input>>,
-    ) -> Result<(), WindowError> {
+    fn add_key_down_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::KeyboardEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -622,8 +593,7 @@ impl Window {
                 let key = event.key();
                 let modifiers = input.modifiers;
                 if let Some(kind) = translate_key(&key) {
-                    input.events.push(Event::Key {
-                        state: State::Pressed,
+                    input.events.push(Event::KeyPress {
                         kind,
                         modifiers,
                         handled: false,
@@ -631,10 +601,7 @@ impl Window {
                     event.stop_propagation();
                     event.prevent_default();
                 }
-                if modifiers.ctrl == State::Released
-                    && modifiers.command == State::Released
-                    && !should_ignore_key(&key)
-                {
+                if !modifiers.ctrl && !modifiers.command && !should_ignore_key(&key) {
                     input.events.push(Event::Text(key));
                     event.stop_propagation();
                     event.prevent_default();
@@ -648,14 +615,14 @@ impl Window {
             .document()
             .unwrap()
             .add_event_listener_with_callback("keydown", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!("Unable to add key down event listener. Error code: {:?}", e),
+            .map_err(|e| {
+                CanvasError::EventListenerFail("keydown".to_string(), format!("{:?}", e))
             })?;
         self.closures_with_keyboardevent.push(closure);
         Ok(())
     }
 
-    fn add_key_up_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<(), WindowError> {
+    fn add_key_up_event_listener(&mut self, input: Rc<RefCell<Input>>) -> Result<()> {
         let closure = Closure::wrap(Box::new(move |event: web_sys::KeyboardEvent| {
             if !event.default_prevented() {
                 let mut input = input.borrow_mut();
@@ -667,8 +634,7 @@ impl Window {
                 }
                 if let Some(kind) = translate_key(&event.key()) {
                     let modifiers = input.modifiers;
-                    input.events.push(Event::Key {
-                        state: State::Released,
+                    input.events.push(Event::KeyRelease {
                         kind,
                         modifiers,
                         handled: false,
@@ -685,9 +651,7 @@ impl Window {
             .document()
             .unwrap()
             .add_event_listener_with_callback("keyup", closure.as_ref().unchecked_ref())
-            .map_err(|e| WindowError::EventListenerError {
-                message: format!("Unable to add key up event listener. Error code: {:?}", e),
-            })?;
+            .map_err(|e| CanvasError::EventListenerFail("keyup".to_string(), format!("{:?}", e)))?;
         self.closures_with_keyboardevent.push(closure);
         Ok(())
     }
@@ -706,6 +670,7 @@ struct Input {
     modifiers: Modifiers,
     last_position: Option<(i32, i32)>,
     last_zoom: Option<f64>,
+    mouse_pressed: Option<MouseButton>,
 }
 
 impl Input {
@@ -718,6 +683,7 @@ impl Input {
             modifiers: Modifiers::default(),
             last_position: None,
             last_zoom: None,
+            mouse_pressed: None,
         }))
     }
 
@@ -747,26 +713,10 @@ impl Input {
 fn update_modifiers(modifiers: &mut Modifiers, event: &web_sys::KeyboardEvent) -> bool {
     let old = modifiers.clone();
     *modifiers = Modifiers {
-        alt: if event.alt_key() {
-            State::Pressed
-        } else {
-            State::Released
-        },
-        ctrl: if event.ctrl_key() {
-            State::Pressed
-        } else {
-            State::Released
-        },
-        shift: if event.shift_key() {
-            State::Pressed
-        } else {
-            State::Released
-        },
-        command: if event.ctrl_key() || event.meta_key() {
-            State::Pressed
-        } else {
-            State::Released
-        },
+        alt: event.alt_key(),
+        ctrl: event.ctrl_key(),
+        shift: event.shift_key(),
+        command: event.ctrl_key() || event.meta_key(),
     };
     old.alt != modifiers.alt
         || old.ctrl != modifiers.ctrl
